package spam.evaluation;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.sparql.algebra.Op;
import com.hp.hpl.jena.sparql.algebra.Table;
import com.hp.hpl.jena.sparql.algebra.TableFactory;
import com.hp.hpl.jena.sparql.core.DatasetGraph;
import com.hp.hpl.jena.sparql.core.Substitute;
import com.hp.hpl.jena.sparql.engine.ExecutionContext;
import com.hp.hpl.jena.sparql.engine.Plan;
import com.hp.hpl.jena.sparql.engine.QueryEngineBase;
import com.hp.hpl.jena.sparql.engine.QueryEngineFactory;
import com.hp.hpl.jena.sparql.engine.QueryEngineRegistry;
import com.hp.hpl.jena.sparql.engine.QueryIterator;
import com.hp.hpl.jena.sparql.engine.binding.Binding;
import com.hp.hpl.jena.sparql.engine.iterator.QueryIteratorCheck;
import com.hp.hpl.jena.sparql.engine.main.QC;
import com.hp.hpl.jena.sparql.engine.ref.EvaluatorFactory;
import com.hp.hpl.jena.sparql.engine.ref.QueryEngineRef;
import com.hp.hpl.jena.sparql.util.Context;

public class QueryEngineExt extends QueryEngineBase {

	public static boolean debug = false;

	public QueryEngineExt(Query query, DatasetGraph dataset, Binding input, Context cxt) {
		super(query, dataset, input, cxt);

	}

	public QueryEngineExt(Op op, DatasetGraph dataset, Binding input, Context cxt) {
		super(op, dataset, input, cxt);

	}

	@Override
	public QueryIterator eval(Op op, DatasetGraph dsg, Binding binding, Context context) {
		if (debug) {
			System.err.println("Using QueryEngineExt");
			EvalExt.debug = true;
		}
		if (binding.vars().hasNext()) {
			op = Substitute.substitute(op, binding);
		}

		ExecutionContext execCxt = new ExecutionContext(context, dsg.getDefaultGraph(), dsg, QC.getFactory(context));
		EvalExt ev = new EvalExt(EvaluatorFactory.create(execCxt));
		Table table = ev.eval(TableFactory.createUnit(), op);
		QueryIterator qIter = table.iterator(execCxt);
		return QueryIteratorCheck.check(qIter, execCxt);
	}

	static public QueryEngineFactory getFactory() {
		return factory;
	}

	static public void register() {
		QueryEngineRegistry.addFactory(factory);
	}

	static public void unregister() {
		QueryEngineRegistry.removeFactory(factory);
	}

	private static QueryEngineFactory factory = new QueryEngineFactory() {

		public boolean accept(Query query, DatasetGraph dataset, Context context) {
			return true;
		}

		public Plan create(Query query, DatasetGraph dataset, Binding binding, Context context) {
			QueryEngineExt engine = new QueryEngineExt(query, dataset, binding, context);
			return engine.getPlan();
		}

		public boolean accept(Op op, DatasetGraph dataset, Context context) {
			return true;
		}

		public Plan create(Op op, DatasetGraph dataset, Binding binding, Context context) {
			QueryEngineRef engine = new QueryEngineRef(op, dataset, binding, context);
			return engine.getPlan();
		}
	};
}
