package spam.evaluation;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.sparql.algebra.Op;
import com.hp.hpl.jena.sparql.algebra.Table;
import com.hp.hpl.jena.sparql.core.DatasetGraph;
import com.hp.hpl.jena.sparql.core.Substitute;
import com.hp.hpl.jena.sparql.engine.*;
import com.hp.hpl.jena.sparql.engine.binding.Binding;
import com.hp.hpl.jena.sparql.engine.iterator.QueryIteratorCheck;
import com.hp.hpl.jena.sparql.engine.main.QC;
import com.hp.hpl.jena.sparql.engine.ref.Evaluator;
import com.hp.hpl.jena.sparql.engine.ref.EvaluatorFactory;
import com.hp.hpl.jena.sparql.engine.ref.QueryEngineRef;
import com.hp.hpl.jena.sparql.util.Context;

/**
 * @author Andres Letelier
 *
 * Implements the Perez et al (ISWC06,
 * http://web.ing.puc.cl/~marenas/publications/iswc06.pdf) top-down evaluation
 * of well designed patterns.
 */
public class QueryEngineWellDesigned extends QueryEngineBase {

    public static boolean debug = false;

    public QueryEngineWellDesigned(Query query, DatasetGraph dataset,
            Binding input, Context cxt) {
        super(query, dataset, input, cxt);

    }

    public QueryEngineWellDesigned(Op op, DatasetGraph dataset, Binding input,
            Context cxt) {
        super(op, dataset, input, cxt);

    }

    @Override
    public QueryIterator eval(Op op, DatasetGraph dsg, Binding binding, Context context) {
        if (debug) {
            System.err.println("Using QueryEngineWellDesigned");
        }
        if (binding.vars().hasNext()) {
            op = Substitute.substitute(op, binding);
        }

        ExecutionContext execCxt = new ExecutionContext(context, dsg.getDefaultGraph(), dsg, QC.getFactory(context));
        Evaluator eval = EvaluatorFactory.create(execCxt);
        Table table = EvalWellDesigned.eval(eval, op);
        QueryIterator qIter = table.iterator(execCxt);
        return QueryIteratorCheck.check(qIter, execCxt);
    }

    static public QueryEngineFactory getFactory() {
        return factory;
    }

    static public void register() {
        QueryEngineRegistry.addFactory(factory);
    }

    static public void unregister() {
        QueryEngineRegistry.removeFactory(factory);
    }
    private static QueryEngineFactory factory = new QueryEngineFactory() {

        public boolean accept(Query query, DatasetGraph dataset, Context context) {
            return true;
        }

        public Plan create(Query query, DatasetGraph dataset, Binding binding, Context context) {
            QueryEngineWellDesigned engine = new QueryEngineWellDesigned(query, dataset, binding, context);
            return engine.getPlan();
        }

        public boolean accept(Op op, DatasetGraph dataset, Context context) {
            return true;
        }

        public Plan create(Op op, DatasetGraph dataset, Binding binding, Context context) {
            QueryEngineRef engine = new QueryEngineRef(op, dataset, binding, context);
            return engine.getPlan();
        }
    };
}
