/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.gui;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;

import spam.exception.PatternTreeFactoryException;
import spam.exception.UntransformableOperatorException;
import spam.model.QueryComponent;
import spam.service.OptNormalFormService;
import spam.service.WellDesignedService;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryParseException;

/**
 * Simplifies adding and removing queries. Singleton
 *
 * @author Andres Letelier
 */
public class QueryListHandler {

	private final DefaultListModel<QueryComponent> model;
	private final DefaultComboBoxModel<QueryComponent> cModel1;
	private final DefaultComboBoxModel<QueryComponent> cModel2;

	private final List<IUpdateAction> updateListener;

	private static QueryListHandler instance;

	public static QueryListHandler getInstance() {
		if (instance == null) {
			instance = new QueryListHandler();
		}
		return instance;
	}

	public DefaultListModel<QueryComponent> getModel() {
		return model;
	}

	/**
	 * @author Sebastian Skritek
	 */
	public DefaultComboBoxModel<QueryComponent> getCModel1() {
		return cModel1;
	}

	/**
	 * @author Sebastian Skritek
	 */
	public DefaultComboBoxModel<QueryComponent> getCModel2() {
		return cModel2;
	}

	private QueryListHandler() {

		this.updateListener = new LinkedList<IUpdateAction>();

		model = new DefaultListModel<QueryComponent>();
		cModel1 = new DefaultComboBoxModel<QueryComponent>();
		cModel2 = new DefaultComboBoxModel<QueryComponent>();
	}

	void registerUpdateListener(IUpdateAction listener) {
		if (!this.updateListener.contains(listener)) {
			this.updateListener.add(listener);
		}
	}

	public void queryGraphUpdated(QueryComponent component) {
		for (IUpdateAction listener : this.updateListener) {
			listener.queryGraphUpdated(component);
		}
	}

	public void queryUpdated(QueryComponent component) {
		for (IUpdateAction listener : this.updateListener) {
			listener.queryChanged(component, true);
		}
	}

	/**
	 * Add a parsed query with a given name to the list
	 *
	 * @param name
	 * @param q
	 * @throws PatternTreeFactoryException
	 */
	public void addQuery(String name, Query q) throws PatternTreeFactoryException {
		// rewriten by SS
		// ORIGINAL LINE:
		// model.addElement(new QueryComponent(name, q));
		// NEW VERSION
		QueryComponent newQ = new QueryComponent(name, q);
		model.addElement(newQ);
		cModel1.addElement(newQ);
		cModel2.addElement(newQ);
	}

	public void removeQuery(QueryComponent qc) {
		model.removeElement(qc);
		cModel1.removeElement(qc);
		cModel2.removeElement(qc);
	}

	/**
	 * @author Sebastian Skritek
	 * @throws PatternTreeFactoryException
	 */
	public void updateQuery(String name, Query q) throws PatternTreeFactoryException {

		for (int i = 0; i < model.getSize(); i++) {
			if (model.get(i).getQueryName().equals(name)) {
				model.get(i).setQuery(q);
			}
		}
	}

	/**
	 * Load a query from a file and add it to the list
	 *
	 * @param f
	 * @throws UntransformableOperatorException
	 * @throws PatternTreeFactoryException
	 */
	public void loadQuery(File f) throws QueryParseException, UntransformableOperatorException, PatternTreeFactoryException {
		// we try to parse the query
		Query q1 = QueryFactory.read(f.getAbsolutePath());
		q1 = OptNormalFormService.getOptNormalForm(q1);
		if (WellDesignedService.isWellDesigned(q1)) {
			q1 = OptNormalFormService.getTreePatternConvertableForm(q1);
		}
		QueryComponent newQ = new QueryComponent(f.getName(), q1, f);
		model.addElement(newQ);
		cModel1.addElement(newQ);
		cModel2.addElement(newQ);

		if (newQ.getTreePattern() == null) {
			throw new PatternTreeFactoryException();
		}
	}

	/**
	 * Returns the list of query names
	 * 
	 * @return
	 */
	public String[] getQueryNames() {
		String[] names = new String[model.size()];
		for (int i = 0; i < model.size(); i++) {
			names[i] = model.get(i).getQueryName();
		}
		return names;
	}

	public void clearList() {
		model.clear();
		cModel1.removeAllElements();
		cModel2.removeAllElements();
	}
}
