/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.UIManager;
import javax.swing.table.TableModel;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.LayoutStyle;

import spam.evaluation.EvalExt;
import spam.exception.PatternTreeFactoryException;
import spam.exception.UntransformableOperatorException;
import spam.model.DatasetComponent;
import spam.model.QueryComponent;
import spam.model.QueryComponent.ResultTableModel;
import spam.model.TreePattern;
import spam.model.WdConflictComponent;
import spam.service.OptNormalFormService;
import spam.service.WellDesignedService;
import spam.transformationRules.R1Application;
import spam.transformationRules.R4Application;
import spam.transformationRules.RuleApplication;
import spam.transformationRules.TransformationUtilities;

import com.hp.hpl.jena.query.ARQ;
import com.hp.hpl.jena.query.Dataset;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryParseException;
import com.hp.hpl.jena.shared.JenaException;
import com.hp.hpl.jena.sparql.algebra.Algebra;
import com.hp.hpl.jena.sparql.algebra.Table;
import com.hp.hpl.jena.sparql.algebra.table.TableN;
import com.hp.hpl.jena.sparql.engine.ExecutionContext;
import com.hp.hpl.jena.sparql.engine.QueryIterator;
import com.hp.hpl.jena.sparql.engine.main.QC;
import com.hp.hpl.jena.sparql.engine.ref.EvaluatorFactory;
import com.hp.hpl.jena.sparql.util.Context;

/**
 *
 * @author Andres Letelier
 */
public class SpamGui extends javax.swing.JFrame implements IUpdateAction, ActionListener {

	private static final long serialVersionUID = 1L;
	private boolean editing = false;
	// GUI elements
	private JPopupMenu listQueriesPopupMenu;
	private JMenuItem lqisave, lqicopy, lqiclose;
	private JFileChooser saveFileChooser;

	/**
	 * Creates new form SpamGuiMain
	 */
	public SpamGui() {

		try {
			// Set System L&F
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Cannot set system look and feel");
			System.err.println(e);
		} finally {
			initComponents();
			manuallyInitComponents();
			QueryListHandler.getInstance().registerUpdateListener(this);
		}
	}

	/**
	 * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The content of this method is always regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		fileChooser = new javax.swing.JFileChooser();
		jLabel2 = new javax.swing.JLabel();
		jButtonNewQuery = new javax.swing.JButton();
		jButtonLoadDataset = new javax.swing.JButton();
		jLabel3 = new javax.swing.JLabel();
		jButtonCreateRandomDataset = new javax.swing.JButton();
		jButtonLoadQuery = new javax.swing.JButton();
		jScrollPane4 = new javax.swing.JScrollPane();
		listQueries = new javax.swing.JList<QueryComponent>();
		jScrollPane5 = new javax.swing.JScrollPane();
		listDatabases = new javax.swing.JList<DatasetComponent>();
		jLabel1 = new javax.swing.JLabel();
		jSplitPane1 = new javax.swing.JSplitPane();
		upperTabbedPane = new javax.swing.JTabbedPane();
		lowerTabbedPane = new javax.swing.JTabbedPane();
		jScrollPane1 = new javax.swing.JScrollPane();
		codeArea = new javax.swing.JTextArea();
		jScrollPane2 = new javax.swing.JScrollPane();
		statisticsArea = new javax.swing.JTextArea();
		jScrollPane3 = new javax.swing.JScrollPane();
		resultsTable = new javax.swing.JTable();
		wdViolationsJScrollPane = new javax.swing.JScrollPane();
		wdViolationsJList = new javax.swing.JList<WdConflictComponent>();
		checkWDButton = new javax.swing.JButton();
		runQueryButton = new javax.swing.JButton();
		jScrollPane8 = new javax.swing.JScrollPane();
		ruleApplicationList = new javax.swing.JList<RuleApplication>();
		jLabel4 = new javax.swing.JLabel();
		jButtonPreviewRule = new javax.swing.JButton();
		jButtonApplyRule = new javax.swing.JButton();
		jLabel6 = new javax.swing.JLabel();
		jPanel1 = new javax.swing.JPanel();
		jLabelRuleDescription = new javax.swing.JLabel();
		openStaticAnalysisButton = new javax.swing.JButton();
		applyChangesButton = new javax.swing.JButton();

		fileChooser.setDialogTitle("Select a query");

		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		setTitle("SPAM: Sparql analysis and manipulation");

		jLabel2.setText("RDF datasets");

		jButtonNewQuery.setText("New");
		jButtonNewQuery.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonNewQueryActionPerformed(evt);
			}
		});

		jButtonLoadDataset.setText("Load");
		jButtonLoadDataset.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonLoadDatasetActionPerformed(evt);
			}
		});

		jLabel3.setText("Queries");

		jButtonCreateRandomDataset.setText("Generate");
		jButtonCreateRandomDataset.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonCreateRandomDatasetActionPerformed(evt);
			}
		});

		jButtonLoadQuery.setText("Load");
		jButtonLoadQuery.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonLoadQueryActionPerformed(evt);
			}
		});

		jScrollPane4.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

		listQueries.setModel(QueryListHandler.getInstance().getModel());
		listQueries.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		listQueries.addMouseListener(new java.awt.event.MouseAdapter() {
			@Override
			public void mouseClicked(java.awt.event.MouseEvent evt) {
				listQueriesMouseClicked(evt);
			}

			@Override
			public void mousePressed(java.awt.event.MouseEvent evt) {
				listQueriesMousePressed(evt);
			}

			@Override
			public void mouseReleased(java.awt.event.MouseEvent evt) {
				listQueriesMouseReleased(evt);
			}
		});
		jScrollPane4.setViewportView(listQueries);

		jScrollPane5.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

		listDatabases.setModel(DatasetListHandler.getInstance().getModel());
		jScrollPane5.setViewportView(listDatabases);

		jLabel1.setText("Editor");

		jSplitPane1.setDividerLocation(250);
		jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

		upperTabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
			@Override
			public void stateChanged(javax.swing.event.ChangeEvent evt) {
				upperTabbedPaneStateChanged(evt);
			}
		});
		jSplitPane1.setTopComponent(upperTabbedPane);

		codeArea.setColumns(20);
		codeArea.setRows(5);
		codeArea.addFocusListener(new java.awt.event.FocusAdapter() {
			@Override
			public void focusGained(java.awt.event.FocusEvent evt) {
				codeAreaFocusGained(evt);
			}

			@Override
			public void focusLost(java.awt.event.FocusEvent evt) {
				codeAreaFocusLost(evt);
			}
		});
		jScrollPane1.setViewportView(codeArea);

		lowerTabbedPane.addTab("Code", jScrollPane1);

		statisticsArea.setColumns(20);
		statisticsArea.setEditable(false);
		statisticsArea.setRows(5);
		jScrollPane2.setViewportView(statisticsArea);

		lowerTabbedPane.addTab("Statistics", jScrollPane2);

		resultsTable.setModel(new javax.swing.table.DefaultTableModel(new Object[][] { {} }, new String[] {

		}));
		jScrollPane3.setViewportView(resultsTable);

		lowerTabbedPane.addTab("Results", jScrollPane3);

		wdViolationsJList.setModel(new DefaultListModel<WdConflictComponent>());
		wdViolationsJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		wdViolationsJList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
			@Override
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				wdViolationsJListValueChanged(evt);
			}
		});
		wdViolationsJScrollPane.setViewportView(wdViolationsJList);

		lowerTabbedPane.addTab("Well-designed violations", wdViolationsJScrollPane);

		jSplitPane1.setBottomComponent(lowerTabbedPane);

		checkWDButton.setText("Check well-designed");
		checkWDButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				checkWDButtonActionPerformed(evt);
			}
		});

		runQueryButton.setText("Run query");
		runQueryButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonRunQueryActionPerformed(evt);
			}
		});

		jScrollPane8.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

		ruleApplicationList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
			@Override
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				ruleApplicationListValueChanged(evt);
			}
		});
		jScrollPane8.setViewportView(ruleApplicationList);

		jLabel4.setText("Rule applications");

		jButtonPreviewRule.setText("Preview");
		jButtonPreviewRule.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonPreviewRuleActionPerformed(evt);
			}
		});

		jButtonApplyRule.setText("Apply");
		jButtonApplyRule.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButtonApplyRuleActionPerformed(evt);
			}
		});

		jLabel6.setText("Rule description");

		jPanel1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

		GroupLayout jPanel1Layout = new GroupLayout(jPanel1);
		jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(GroupLayout.LEADING).add(jLabelRuleDescription, GroupLayout.PREFERRED_SIZE, 273, GroupLayout.PREFERRED_SIZE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(GroupLayout.LEADING).add(
				jPanel1Layout.createSequentialGroup().add(jLabelRuleDescription, GroupLayout.DEFAULT_SIZE, 12, Short.MAX_VALUE).add(153, 153, 153)));

		openStaticAnalysisButton.setText("Static analysis");
		openStaticAnalysisButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				openStaticAnalysisButtonActionPerformed(evt);
			}
		});

		applyChangesButton.setText("Apply");
		applyChangesButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				applyChangesButtonActionPerformed(evt);
			}
		});

		GroupLayout layout = new GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.LEADING).add(
				layout.createSequentialGroup()
						.addContainerGap()
						.add(layout
								.createParallelGroup(GroupLayout.LEADING)
								.add(layout
										.createParallelGroup(GroupLayout.LEADING, false)
										.add(GroupLayout.TRAILING, jScrollPane4)
										.add(GroupLayout.TRAILING,
												layout.createSequentialGroup().add(jButtonNewQuery, GroupLayout.PREFERRED_SIZE, 99, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.RELATED)
														.add(jButtonLoadQuery, GroupLayout.PREFERRED_SIZE, 99, GroupLayout.PREFERRED_SIZE))
										.add(GroupLayout.TRAILING,
												layout.createSequentialGroup().add(jButtonCreateRandomDataset).addPreferredGap(LayoutStyle.RELATED)
														.add(jButtonLoadDataset, GroupLayout.PREFERRED_SIZE, 99, GroupLayout.PREFERRED_SIZE)).add(jScrollPane5)).add(jLabel3).add(jLabel2))
						.addPreferredGap(LayoutStyle.RELATED)
						.add(layout
								.createParallelGroup(GroupLayout.LEADING)
								.add(jSplitPane1, GroupLayout.DEFAULT_SIZE, 469, Short.MAX_VALUE)
								.add(layout
										.createSequentialGroup()
										.add(layout
												.createParallelGroup(GroupLayout.LEADING)
												.add(jLabel1)
												.add(layout.createSequentialGroup().add(applyChangesButton).addPreferredGap(LayoutStyle.RELATED).add(runQueryButton)
														.addPreferredGap(LayoutStyle.RELATED).add(checkWDButton).addPreferredGap(LayoutStyle.RELATED).add(openStaticAnalysisButton)))
										.add(0, 78, Short.MAX_VALUE)))
						.addPreferredGap(LayoutStyle.RELATED)
						.add(layout
								.createParallelGroup(GroupLayout.LEADING, false)
								.add(jLabel4)
								.add(jScrollPane8)
								.add(jLabel6)
								.add(layout.createSequentialGroup().add(jButtonPreviewRule).addPreferredGap(LayoutStyle.RELATED)
										.add(jButtonApplyRule, GroupLayout.PREFERRED_SIZE, 88, GroupLayout.PREFERRED_SIZE))
								.add(jPanel1, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)).addContainerGap()));
		layout.setVerticalGroup(layout.createParallelGroup(GroupLayout.LEADING).add(
				layout.createSequentialGroup()
						.addContainerGap()
						.add(layout
								.createParallelGroup(GroupLayout.LEADING)
								.add(layout
										.createSequentialGroup()
										.add(layout.createParallelGroup(GroupLayout.LEADING).add(jLabel4).add(GroupLayout.TRAILING, jLabel3))
										.addPreferredGap(LayoutStyle.RELATED)
										.add(layout.createParallelGroup(GroupLayout.LEADING).add(jScrollPane4, GroupLayout.DEFAULT_SIZE, 180, Short.MAX_VALUE)
												.add(jScrollPane8, GroupLayout.PREFERRED_SIZE, 174, GroupLayout.PREFERRED_SIZE))
										.add(layout
												.createParallelGroup(GroupLayout.LEADING)
												.add(layout.createSequentialGroup().add(layout.createParallelGroup(GroupLayout.BASELINE).add(jButtonLoadQuery).add(jButtonNewQuery))
														.addPreferredGap(LayoutStyle.RELATED).add(jLabel2).addPreferredGap(LayoutStyle.RELATED)
														.add(jScrollPane5, GroupLayout.DEFAULT_SIZE, 167, Short.MAX_VALUE))
												.add(layout.createSequentialGroup().addPreferredGap(LayoutStyle.RELATED)
														.add(layout.createParallelGroup(GroupLayout.BASELINE).add(jButtonPreviewRule).add(jButtonApplyRule)).addPreferredGap(LayoutStyle.RELATED)
														.add(jLabel6).addPreferredGap(LayoutStyle.RELATED).add(jPanel1, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
								.add(layout.createSequentialGroup().add(jLabel1).addPreferredGap(LayoutStyle.RELATED).add(jSplitPane1, GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)))
						.add(2, 2, 2)
						.add(layout.createParallelGroup(GroupLayout.BASELINE).add(jButtonLoadDataset).add(jButtonCreateRandomDataset).add(runQueryButton).add(checkWDButton)
								.add(openStaticAnalysisButton).add(applyChangesButton)).addContainerGap()));
		getContentPane().setPreferredSize(new Dimension(1000, 600));
		pack();
	}// </editor-fold>//GEN-END:initComponents

	private void manuallyInitComponents() {

		this.wdViolationsJLabel = new JLabel("The query is well-designed");

		// The popop menu for the list of queries
		listQueriesPopupMenu = new JPopupMenu();
		listQueriesPopupMenu.add(lqisave = new JMenuItem("Save ..."));
		listQueriesPopupMenu.add(new JPopupMenu.Separator());
		listQueriesPopupMenu.add(lqicopy = new JMenuItem("Copy"));
		listQueriesPopupMenu.add(new JPopupMenu.Separator());
		listQueriesPopupMenu.add(lqiclose = new JMenuItem("Close"));
		lqisave.addActionListener(this);
		lqicopy.addActionListener(this);
		lqiclose.addActionListener(this);

		this.noRulesJLabel = new JLabel("<html>The query is not well-designed,<br/>" + "therefore no rules are applicable</html>");

	}

	private void jButtonNewQueryActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonNewQueryActionPerformed

		// TODO: it would be nice if we could somehow react here to the fact
		// that we have unsaved changes on the current query.

		String name = userInputNewQueryName();
		if (name == null) {
			return;
		}

		Query q = QueryFactory.create("select * where {?x ?y ?z}");
		try {
			QueryListHandler.getInstance().addQuery(name, q);
		} catch (PatternTreeFactoryException e) {
			// This should be impossible to fail
		}
	}// GEN-LAST:event_jButtonNewQueryActionPerformed

	/**
	 * Retrieves a new query name from a user.
	 *
	 * Displays a dialog to the user asking for a new query name. This dialog is shown repeatedly until either the user aborts or enters a name not assigned to any query yet.
	 *
	 * @return the name chosen by the user
	 */
	private String userInputNewQueryName() {

		String name = "";
		while (name.equals("")) {
			name = (String) JOptionPane.showInputDialog(this, "Insert a name for the query", "Query name", JOptionPane.PLAIN_MESSAGE, null, null, "");
			if (name == null) {
				return null;
			}
			name = name.trim();
			for (String q : QueryListHandler.getInstance().getQueryNames()) {
				if (name.equals(q)) {
					System.err.println("Name " + name + " already exists");
					name = "";
					break;
				}
			}
		}
		return name;
	}

	private void jButtonLoadDatasetActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonLoadDatasetActionPerformed
		fileChooser.setDialogTitle("Select database file");
		if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
			return;
		}
		File f = fileChooser.getSelectedFile();
		try {
			DatasetListHandler.getInstance().loadDataset(f);
		} catch (JenaException ex) {
			System.out.println(ex.getMessage());
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, "Unsupported format for " + f.getPath());
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "Something went wrong trying to open " + f.getPath());
		}

	}// GEN-LAST:event_jButtonLoadDatasetActionPerformed

	private String sp2bPath = null;

	private void jButtonCreateRandomDatasetActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonCreateRandomDatasetActionPerformed
		if (sp2bPath == null) {
			fileChooser.setDialogTitle("SP2B executable");
			if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
				return;
			}
			File f = fileChooser.getSelectedFile();
			try {
				sp2bPath = f.getCanonicalPath().toString();
				sp2bPath = sp2bPath.substring(0, sp2bPath.length() - "/sp2b_gen".length());
				// System.out.println(sp2bPath);
			} catch (IOException ex) {
				JOptionPane.showMessageDialog(this, "Something went wrong trying to open " + f.getPath());
				return;
			}
		}
		String s = JOptionPane.showInputDialog(this, "Select number of triples", 100);
		String of = "/sp2b_";
		int n;
		try {
			n = Integer.parseInt(s);
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "You must enter a number");
			return;
		}

		try {

			of += n + ".n3";
			System.out.println("Running " + sp2bPath + "/sp2b_gen -t " + n + " " + sp2bPath + of);

			Runtime.getRuntime().exec("./sp2b_gen -t " + n + " " + sp2bPath + of, null, new File(sp2bPath));
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(this, "Something went wrong trying to run \"" + sp2bPath + "/sp2b_gen -t " + n + "\"");
			return;
		}
		try {
			File f = new File(sp2bPath + of);
			System.out.println("Opoening: " + sp2bPath + of);
			DatasetListHandler.getInstance().loadDataset(f);
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "Something went wrong trying to open \"" + sp2bPath + of + "\"");
		}
	}// GEN-LAST:event_jButtonCreateRandomDatasetActionPerformed

	private void jButtonPreviewRuleActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonPreviewRuleActionPerformed

		int rule = this.ruleApplicationList.getSelectedIndex();

		if (rule == -1) {
			return;
		}

		QueryComponent query = getActiveQueryComponent();
		if (query == null) {
			return;
		}

		TreePattern clone = query.getTreePattern().clone();
		RuleApplication r = TransformationUtilities.getApplicableRules(clone).get(rule);
		r.apply();

		QueryComponent component = new QueryComponent(query.getName(), clone);
		component.setPrefixMapping(query.getQuery().getPrefixMapping());
		JFrame show = new JFrame("Preview");
		show.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		show.add(component, BorderLayout.CENTER);

		show.setSize(query.getGraphicalTree().getPreferredSize());
		show.setVisible(true);

	}// GEN-LAST:event_jButtonPreviewRuleActionPerformed

	/**
	 * Action Handler for the "static analysis" button.
	 *
	 * Brings the SubsumptionGui to the front
	 *
	 * @param evt
	 */
	private void openStaticAnalysisButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButton9ActionPerformed

		// TODO: it would be nice if we could somehow react if there are
		// unsaved changes for the currently active query.

		if (QueryListHandler.getInstance().getModel().size() < 2) {
			// Nothing todo, so tell the user that it is rather useless
			// (note that if the subsumption screen is loaded without any query
			// present we get a null pointer exception nevertheless)
			JOptionPane.showMessageDialog(this, "Please load at least two queries " + "to compare - this cannot be done in the subsumption screen.");
			return;
		}

		SubsumptionGui sub = new SubsumptionGui();
		sub.setVisible(true);
	}// GEN-LAST:event_jButton9ActionPerformed

	/**
	 * Probably deprecated.
	 *
	 * Although I like this function and the output it gives, in the current model it does not fit, because the check is done automatically.
	 *
	 * @param evt
	 */
	@Deprecated
	private void checkWDButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_checkWDButtonActionPerformed

		// If no query is selected, then just tell this to the user and abort
		if (getActiveQueryComponent() == null) {
			JOptionPane.showMessageDialog(this, "No query selected", "Result of well-designed check", JOptionPane.INFORMATION_MESSAGE);

			return;
		}

		if (this.editing) {
			if (!this.saveCodeArea()) {
				JOptionPane.showMessageDialog(this, "Cannot check the query because of some syntax error", "Result of well-designed check", JOptionPane.INFORMATION_MESSAGE);

				return;
			}
		}

		// otherwise get the results:
		String optnf;
		String wdCheckResult, wd2CheckResult;
		String qwdCheckresult;
		TreePattern patternTree = null;

		// check if the query is in OPT-normal form
		if (OptNormalFormService.isOptNormalForm(getActiveQuery())) {
			optnf = "<font color=\"#00aa00\">The query is in OPT-normal form</font>";

			patternTree = getActiveQueryComponent().getTreePattern();
		} else {
			optnf = "<font color=\"#aa0000\">The query is not in OPT-normal form</font>";
		}

		// check if the query is indeed well-designed

		if (patternTree != null) {
			if (WellDesignedService.isWellDesigned(getActiveQuery())) {
				wdCheckResult = "<font color=\"#00aa00\">The query is well-designed</font>";
			} else {
				wdCheckResult = "<font color=\"#aa0000\">The query is not well-designed</font>";
			}
		} else {
			wdCheckResult = "<font color=\"#aa0000\">Since the query is not in " + "OPT-normal form, well-designedness of the query cannot be checked.</font>";
		}
		if (patternTree != null) {
			if (patternTree.checkWellDesigned()) {
				wd2CheckResult = "<font color=\"#00aa00\">The Pattern Tree is well-designed</font>";
				qwdCheckresult = "<font color=\"#00aa00\">The Pattern Tree is quasi well-designed</font>";
			} else {
				wd2CheckResult = "<font color=\"#aa0000\">The Pattern Tree is not well-designed</font>";
				// but maybe it is quasi well-designed
				if (patternTree.checkQuasiWellDesigned()) {
					qwdCheckresult = "<font color=\"#00aa00\">The Pattern Tree is quasi well-designed</font>";
				} else {
					qwdCheckresult = "<font color=\"#aa0000\">The Pattern Tree is not quasi well-designed</font>";
				}
			}
		} else {
			wd2CheckResult = "<font color=\"#aa0000\">Since the query is not in " + "OPT-normal form, well-designedness of QWDPT cannot be checked.</font>";
			qwdCheckresult = "<font color=\"#aa0000\">Since the query is not in " + "OPT-normal form, quasi-well-designedness of QWDPT cannot be checked.</font>";
		}

		String outputString = "<html>Result for query " + getActiveQueryComponent().getQueryName() + "<br/>" + optnf + "<br/>" + wdCheckResult + "<br/>" + wd2CheckResult + "<br/>" + qwdCheckresult
				+ "</html>";

		// if the query is not well designed or not quasi well designed, we have
		// to show and initialize the list of counterexamples
		prepareQwdTab(patternTree);

		// show the user the results of our tests
		JOptionPane.showMessageDialog(this, outputString, "Result of well-designed check", JOptionPane.INFORMATION_MESSAGE);
	}// GEN-LAST:event_checkWDButtonActionPerformed

	/**
	 * Sets the output on the tab for the results of the well-designed tab.
	 *
	 * If the query is not in OPT normal form, a corresponding error message is displayed. If the query is well designed, a corresponding message is displayed. If the query is not well designed, a
	 * list of violations is offered.
	 *
	 * @param qwdpt
	 *            If null, then the query is not in OPT normal form
	 */
	private void prepareQwdTab(TreePattern qwdpt) {

		if (qwdpt == null) {

			wdViolationsJLabel.setText("<html>The query is not in OPT-normal form.<br/>" + "Therefor it cannot be checked to be well-designed.</html>");
			this.wdViolationsJScrollPane.setViewportView(this.wdViolationsJLabel);

		} else if (qwdpt.isWellDesigned()) {

			wdViolationsJLabel.setText("The query is well-designed");
			this.wdViolationsJScrollPane.setViewportView(this.wdViolationsJLabel);

		} else {

			DefaultListModel<WdConflictComponent> model = (DefaultListModel<WdConflictComponent>) this.wdViolationsJList.getModel();
			model.removeAllElements();
			for (String s : qwdpt.getWdCounterexampleVars()) {
				model.addElement(new WdConflictComponent(qwdpt.getWdConflict(s)));
			}
			this.wdViolationsJScrollPane.setViewportView(this.wdViolationsJList);
		}

	}

	private void checkActiveQueryForWd() {

		TreePattern qwdpt = null;

		if (OptNormalFormService.isOptNormalForm(getActiveQuery())) {
			qwdpt = getActiveQueryComponent().getTreePattern();
			qwdpt.checkWellDesigned();
		}

		// display the results of the test
		prepareQwdTab(qwdpt);

	}

	private void jButtonRunQueryActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonRunQueryActionPerformed
		Dataset data;
		try {
			data = listDatabases.getSelectedValue().getDataset();
		} catch (NullPointerException e) {
			JOptionPane.showMessageDialog(this, "No dataset selected");
			return;
		}

		// I have no idea of what the next two lines do, but they work.
		Context cont = ARQ.getContext().copy();
		ExecutionContext execCxt = new ExecutionContext(cont, data.asDatasetGraph().getDefaultGraph(), data.asDatasetGraph(), QC.getFactory(cont));

		EvalExt ev = new EvalExt(EvaluatorFactory.create(execCxt));

		QueryComponent q;

		String statistics = "";
		long timeARQ = 0, timeExt = 0;
		Table res;
		try {
			long begintime, endtime;
			// changed by Sebastian Skritek
			// q = (QueryComponent) listQueries.getSelectedValue();
			q = getActiveQueryComponent();

			begintime = System.currentTimeMillis();

			res = ev.eval(q.getTreePattern());

			endtime = System.currentTimeMillis();
			timeExt = endtime - begintime;
			statistics += "Extension engine:\n";
			statistics += "Total running time: " + timeExt + "ms\n";
			statistics += "Number of results: " + res.size() + "\n\n";

			if (WellDesignedService.isWellDesigned(q.getQuery())) {

			} else {
				statistics += "=== Query is not well-designed!! Semantics of evaluation differs!! ===\n\n";
			}
			begintime = System.currentTimeMillis();
			// this method only creates the iterator; we need to actually fetch
			// all the results to get the real running time

			QueryIterator qiter = Algebra.exec(q.getTreePattern().toOPNoHead(), data);

			// now we actually fetch the results and create a table from them
			res = new TableN(qiter);
			endtime = System.currentTimeMillis();

			timeARQ = endtime - begintime;
			statistics += "ARQ main engine:\n";
			statistics += "Total running time: " + timeARQ + "ms\n";
			statistics += "Number of results: " + res.size();

			q.updateGraph();

			q.setStatisticsString(statistics);

			statisticsArea.setText(statistics);
			lowerTabbedPane.setSelectedIndex(1);

			q.setResultTableModel(res);

			// check if we have too many results
			TableModel model = q.getResultTableModel();
			if (model instanceof ResultTableModel) {
				ResultTableModel resultModel = (ResultTableModel) model;
				if (!resultModel.containsAllResults()) {
					JOptionPane.showMessageDialog(this, "Too many results, " + "only the first " + resultModel.getRowCount() + " rows" + " are shown.");
				}
			}

			resultsTable.setModel(q.getResultTableModel());

		} catch (NullPointerException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(this, "No query selected");
		}
	}// GEN-LAST:event_jButtonRunQueryActionPerformed

	/**
	 * Show a file chooser dialog to load a query
	 *
	 * @param evt
	 */
	private void jButtonLoadQueryActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonLoadQueryActionPerformed
		fileChooser.setDialogTitle("Select query file");
		if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
			return;
		}
		File f = fileChooser.getSelectedFile();
		try {
			QueryListHandler.getInstance().loadQuery(f);
		} catch (UntransformableOperatorException ex) {
			JOptionPane.showMessageDialog(this, "The given query was not and could not be transformed into Opt Normal Form", "Auto Transformation Failed", JOptionPane.ERROR_MESSAGE);
		} catch (PatternTreeFactoryException ex) {
			JOptionPane.showMessageDialog(this, "The given query has no valid pattern tree representation", "Pattern Tree Creation Failed", JOptionPane.ERROR_MESSAGE);
		} catch (QueryParseException ex) {
			JOptionPane.showMessageDialog(this, "Could not parse query: " + ex.getMessage());
			System.err.println(ex);
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "Something went wrong trying to open " + f.getPath());
		}
	}// GEN-LAST:event_jButtonLoadQueryActionPerformed

	/**
	 * Open panes for each query if it's not currently opened
	 *
	 * @param evt
	 */
	private void listQueriesMouseClicked(java.awt.event.MouseEvent evt) {// GEN-FIRST:event_listQueriesMouseClicked

		// first of all check if we have a popup action or something else
		if (evt.isPopupTrigger()) {
			// this is handled elsewhere, just ignore
			return;
		}

		// TODO: it would be nice if we could somehow store unsaved changes
		// on the query code, instead of just throwing them away

		// bring the current query to the front
		QueryComponent query = listQueries.getSelectedValue();

		if (query == null) {
			return;
		}

		query.updateViewportView();

		// check if query already displayed, add if not
		if (upperTabbedPane.indexOfComponent(query) < 0) {
			upperTabbedPane.add(query.getQueryName(), query);
			int i = upperTabbedPane.getTabCount() - 1;
			upperTabbedPane.setTabComponentAt(i, new ButtonTabComponent(upperTabbedPane));
		}

		// activate
		upperTabbedPane.setSelectedComponent(query);

		// upperTabbedPane.set
	}// GEN-LAST:event_listQueriesMouseClicked

	/**
	 * Update the code area whenever we change tabs
	 *
	 * @param evt
	 */
	private void upperTabbedPaneStateChanged(javax.swing.event.ChangeEvent evt) {// GEN-FIRST:event_upperTabbedPaneStateChanged

		QueryComponent query = getActiveQueryComponent();

		if (query == null) {

			// no more query open, hence reset all fields
			activeQueryChanged(false);
			return;
		}

		query.updateViewportView();

		String text = "";
		if (query != null) {
			if (query.hasUnsavedChanges()) {
				text = query.getUnsavedQueryString();
			} else {
				if (query.getQuery() != null) {
					text = query.getQuery().toString();
				}
			}
			codeArea.setText(text);
			this.activeQueryChanged(false);
		}

		statisticsArea.setText(query.getStatisticsString());
	}// GEN-LAST:event_upperTabbedPaneStateChanged

	/**
	 * Tries to parse the string currently present in the codeArea textbox.
	 *
	 * Tries to parse the query (by handing it over to Jena). If it works, it returns a query object, otherwise it shows a Dialog that informs about the error.
	 *
	 * @param queryS
	 *            the string that shall be parsed
	 * @return the query if successful, null otherwise
	 */
	private Query parseQuery(String queryS) {

		Query newQ;
		try {
			newQ = QueryFactory.create(queryS);
		} catch (QueryParseException qpe) {
			JOptionPane.showMessageDialog(this, "Could not parse query: " + qpe.getMessage());
			return null;
		}
		return newQ;
	}

	/**
	 * Updates the currently active query component by setting the query to the string currently present in the code Area.
	 *
	 * If the query String represents a valid query, this query is saved in the currently active query component. Further actions to update the graphical editor and to invalidate all kinds of results
	 * are then called
	 *
	 * @return true if the update was successful, false otherwise
	 */
	private boolean saveCodeArea() {

		if (!this.editing) {
			return true;
		}

		// First check if the query is syntactic correct
		Query newQ = parseQuery(codeArea.getText());
		if (newQ == null) {
			codeArea.requestFocus();
			return false;
		}

		if (!OptNormalFormService.isOptNormalForm(newQ)) {
			try {
				// try to convert it into opt - normal form
				newQ = OptNormalFormService.getOptNormalForm(newQ);
				codeArea.setText(newQ.toString());
			} catch (UntransformableOperatorException e) {
				JOptionPane.showMessageDialog(this, "The given query was not and could not be transformed into Opt Normal Form", "Auto Transformation Failed", JOptionPane.ERROR_MESSAGE);
			}
		}
		if (WellDesignedService.isWellDesigned(newQ)) {
			try {
				newQ = OptNormalFormService.getTreePatternConvertableForm(newQ);
			} catch (UntransformableOperatorException e) {
				JOptionPane.showMessageDialog(this, "The given query could not be transformed to be convertable to a pattern tree", "Auto Transformation Failed", JOptionPane.ERROR_MESSAGE);
			}
		}

		codeArea.setText(newQ.toString());
		try {
			getActiveQueryComponent().setQuery(newQ);
		} catch (PatternTreeFactoryException e) {
			JOptionPane.showMessageDialog(this, "The given query has no vaild pattern tree representation", "Pattern Tree Creation Failed", JOptionPane.ERROR_MESSAGE);
		}

		this.editing = false;
		return true;
	}

	private void listQueriesMousePressed(java.awt.event.MouseEvent evt) {// GEN-FIRST:event_listQueriesMousePressed

		if (evt.isPopupTrigger()) {
			this.showListQueriesPopupMenu(evt);
		}
	}// GEN-LAST:event_listQueriesMousePressed

	private void listQueriesMouseReleased(java.awt.event.MouseEvent evt) {// GEN-FIRST:event_listQueriesMouseReleased

		if (evt.isPopupTrigger()) {
			this.showListQueriesPopupMenu(evt);
		}

	}// GEN-LAST:event_listQueriesMouseReleased

	private void jButtonApplyRuleActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jButtonApplyRuleActionPerformed
		// apply a rule
		RuleApplication rule = this.ruleApplicationList.getSelectedValue();
		QueryComponent q = getActiveQueryComponent();

		if (rule == null || q == null) {
			return;
		}

		// rules 1 and 4 must only be applied if the result is well designed
		// otherwise, we just highlight the nodes/triples
		if (rule instanceof R1Application || rule instanceof R4Application) {

			// if(rule.checkResultIsWellDesigned(q,this.ruleApplicationList.getSelectedIndex()))
			// {
			if (rule.checkResultIsWellDesigned(q)) {
				rule.apply();
			} else {
				if (rule instanceof R1Application) {
					// go mark the triple
					rule.apply();
				} else {
					System.err.println("Sorry, this rule cannot be applied. Result would not be well-designed");
				}
			}

		} else {
			rule.apply();
		}
		q.updateGraph();
		q.setRuleApplications(new RuleApplicationList(q));

		getActiveQueryComponent().syncFromGraph();
	}// GEN-LAST:event_jButtonApplyRuleActionPerformed

	/**
	 * The users wants to commit the manual changes to the query string.
	 *
	 * So we have to call the respective methods ...
	 *
	 * @param evt
	 */
	private void applyChangesButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_applyChangesButtonActionPerformed

		if (getActiveQueryComponent() == null) {
			return; // we just ignore this
		}
		// otherwise, all we have to do is to save the query
		this.saveCodeArea();
	}

	private void wdViolationsJListValueChanged(javax.swing.event.ListSelectionEvent evt) {// GEN-FIRST:event_wdViolationsJListValueChanged

		WdConflictComponent model = WdConflictComponent.wdSelectionChanged(this.wdViolationsJList.getSelectedValue());

		if (model == null) {
			// nothing todo
			return;
		}

		// remove the old highlightings
		// add the new highlightings
		getActiveQueryComponent().setWdConflictHighlightVar(model.getConflict());
		getActiveQueryComponent().repaint();
	}

	/**
	 * At the moment this function is not implemented.
	 *
	 * The goal is that this function is used instead of the Apply-Button.
	 *
	 * @param evt
	 */
	private void codeAreaFocusLost(java.awt.event.FocusEvent evt) {// GEN-FIRST:event_codeAreaFocusLost
		// TODO: in a perfect world we would now check if the query has changed
		// and ask the user if we shall save or reset the query before we do
		// anything else.
		// However, we do not live in a perfect world and we do not manage to
		// do anything of this ...

		/*
		 * if (this.editing) { //otherwise the changes have already been saved due to an earlier event saveCodeArea(); }
		 */
	}

	/**
	 * Event Handler for the codeArea: called whenever it gains the focus.
	 *
	 * Sets the editing flag to true, i.e. indicates that the query is currently been edited.
	 *
	 * @param evt
	 */
	private void codeAreaFocusGained(java.awt.event.FocusEvent evt) {// GEN-FIRST:event_codeAreaFocusGained
		this.editing = true;
	}// GEN-LAST:event_codeAreaFocusGained

	private void ruleApplicationListValueChanged(javax.swing.event.ListSelectionEvent evt) {// GEN-FIRST:event_ruleApplicationListValueChanged
		RuleApplication r = ruleApplicationList.getSelectedValue();
		if (r == null) {
			jLabelRuleDescription.setText("");
		} else {
			jLabelRuleDescription.setText(r.getDescription());
			// check if some node needs to be highlighted
			r.highlightNode(getActiveQueryComponent());
		}
	}// GEN-LAST:event_ruleApplicationListValueChanged

	private void showListQueriesPopupMenu(java.awt.event.MouseEvent e) {

		int ind = this.listQueries.locationToIndex(e.getPoint());
		if (ind < 0) {
			return;
		}

		Rectangle listItem = this.listQueries.getCellBounds(ind, ind);
		if (!listItem.contains(e.getPoint()) || ind != this.listQueries.getSelectedIndex()) {
			return;
		}

		listQueriesPopupMenu.show(listQueries, e.getX(), e.getY());
	}

	@Override
	public void actionPerformed(ActionEvent ae) {

		if (ae.getSource() == lqicopy) {
			// ask for name and copy the currently selected item
			this.copyActiveQuery();
		} else if (ae.getSource() == lqisave) {
			// save the currently selected item
			this.saveActiveQuery();
		} else if (ae.getSource() == lqiclose) {
			// close the currently selected query
			this.closeActiveQuery();
		}
	}

	/**
	 * returns the QueryComponent of the query currently selected in the list left.
	 *
	 * Note that getActiveQueryComponent() returns the component to the query fitting to the currently shown tab.
	 *
	 * @return
	 */
	private QueryComponent getSelectedQueryComponent() {
		return listQueries.getSelectedValue();
	}

	/**
	 * Returns the Query object of the query currently selected in the list left.
	 *
	 * Note that getActiveQuery() returns the component to the query fitting to the currently active tab, while this methods returns the element selected in the list left.
	 *
	 * @return
	 */
	private Query getSelectedQuery() {
		return listQueries.getSelectedValue().getQuery();
	}

	private void saveActiveQuery() {

		File targetFile = this.getSaveFile("Save Query", "save", "save to selected file", 's', getSelectedQueryComponent().getQueryFile());
		getSelectedQueryComponent().saveQueryToFile(targetFile);
	}

	private void copyActiveQuery() {

		// first get a new name for the query
		String name = this.userInputNewQueryName();

		// then create the query
		Query q = QueryFactory.create(getSelectedQuery().serialize());
		try {
			QueryListHandler.getInstance().addQuery(name, q);
		} catch (PatternTreeFactoryException e) {
			JOptionPane.showMessageDialog(this, "The corresponding pattern tree to this query could not be built", "Pattern Tree Creation Failed", JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * removes the selected query from the query list at the left and from the tabbed pane.
	 *
	 * The query is not saved if unsaved, but just removed.
	 */
	private void closeActiveQuery() {

		// remove fromm the tabbed Pane
		upperTabbedPane.remove(getSelectedQueryComponent());

		// just remove the query
		QueryListHandler.getInstance().removeQuery(getSelectedQueryComponent());
	}

	private File getSaveFile(String title, String approveButText, String approveButTooltip, char approveButShort, File curFile) {

		if (this.saveFileChooser == null) {
			saveFileChooser = new JFileChooser();
		}
		saveFileChooser.setDialogTitle(title);
		saveFileChooser.setApproveButtonText(approveButText);
		saveFileChooser.setApproveButtonToolTipText(approveButTooltip);
		saveFileChooser.setApproveButtonMnemonic(approveButShort);
		saveFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		saveFileChooser.rescanCurrentDirectory();
		saveFileChooser.setSelectedFile(curFile);

		int result = saveFileChooser.showOpenDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {
			File file = saveFileChooser.getSelectedFile();
			if (file.exists() && !askOverwriteFile(file.getName())) {
				return null;
			}
			return file;
		} else {
			return null;
		}
	}

	/**
	 * A small utility function that asks the user if an already existing file should be overwritten.
	 *
	 * @param filename
	 *            the filename of the file (to be displayed to the user)
	 * @return true if the user wants the file to be overwritten, false otherwise
	 */
	private boolean askOverwriteFile(String filename) {

		Object[] options = { "Overwrite", "Abort" };
		int ans = JOptionPane.showOptionDialog(this, "The file " + filename + " already exists." + "What do you want to do?", "File already exists", JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE, null, options, options[0]);

		return ans == 0;
	}

	/**
	 * @author Sebastian Skritek needs to checked if we do not want to determine the "active" query in a slightly differnt way
	 * 
	 * @return returns the QueryComponent current selected as "active", i.e. the component we suppose the user is currently interested in currently, we assume the component to be currently selected to
	 *         be the the query that is currently selected in the tab-pane
	 */
	private QueryComponent getActiveQueryComponent() {
		return (QueryComponent) upperTabbedPane.getSelectedComponent();
	}

	/**
	 * @return the Query current selected as "active", i.e. the query we suppose the user is currently interested in currently, we assume the query to be currently selected to be the the query that is
	 *         currently selected in the tab-pane
	 */
	private Query getActiveQuery() {
		return ((QueryComponent) upperTabbedPane.getSelectedComponent()).getQuery();
	}

	@Override
	public void queryGraphUpdated(QueryComponent qc) {
		if (qc == this.getActiveQueryComponent()) {
			codeArea.setText(qc.getQuery().toString());
		}
	}

	/**
	 * Major hook for all actions that must be made once the query has changed.
	 *
	 * This includes stuff like checking if the query is well-designed (or invalidating the new results) , checking rule application, ...
	 *
	 * The function should be called whenever a new query is selected, or there have been changes on the current query.
	 */
	@Override
	public void queryChanged(QueryComponent qc, boolean edited) {
		if (qc == getActiveQueryComponent()) {
			this.activeQueryChanged(edited);
		}
	}

	/**
	 * Major hook for all actions that must be made once the query has changed.
	 *
	 * This includes stuff like checking if the query is well-designed (or invalidating the new results) , checking rule application, ...
	 *
	 * The function should be called whenever a new query is selected, or there have been changes on the current query.
	 */
	public void activeQueryChanged(boolean edited) {

		if (getActiveQueryComponent() == null) {
			this.resetFields();
			return;
		}

		// deactivate controls if no pattern tree is present
		if (this.getActiveQueryComponent().getTreePattern() == null) {
			this.hideRuleApplications();
			this.checkWDButton.setEnabled(false);
			this.runQueryButton.setEnabled(false);
			this.openStaticAnalysisButton.setEnabled(false);
			return;
		} else {
			this.checkWDButton.setEnabled(true);
			this.runQueryButton.setEnabled(true);
			this.openStaticAnalysisButton.setEnabled(true);
		}

		// invalidate the runtime information for the last run (if edited)
		if (edited) {
			this.getActiveQueryComponent().getTreePattern().unsetRuntimeStatistics();
		}

		// check for rule applications
		// only do this if the query is well-designed, otherwise just show
		// a hint that this does not work

		boolean isWellDesigned = false;

		if (OptNormalFormService.isOptNormalForm(getActiveQuery())) {
			TreePattern qwdpt = getActiveQueryComponent().getTreePattern();
			isWellDesigned = qwdpt.checkWellDesigned();
		}

		if (isWellDesigned) {
			this.showRuleApplications();
		} else {
			this.hideRuleApplications();
		}

		// check for the query being well-designed
		checkActiveQueryForWd();
	}

	private void resetFields() {

		// clear the rule application lilst
		this.jScrollPane8.setViewportView(this.ruleApplicationList);
		ruleApplicationList.setModel(new DefaultListModel<RuleApplication>());

		// reset the rule explanation
		this.jLabelRuleDescription.setText("");

		// clear the textual representation of the query
		this.codeArea.setText("");

		// clear the statistical information
		this.statisticsArea.setText("");

		// clear the query results

		// and clear the well-designedness tab
		DefaultListModel<WdConflictComponent> model = (DefaultListModel<WdConflictComponent>) this.wdViolationsJList.getModel();
		model.removeAllElements();
		this.wdViolationsJScrollPane.setViewportView(this.wdViolationsJList);
	}

	private void showRuleApplications() {

		this.jLabelRuleDescription.setText("");
		this.jScrollPane8.setViewportView(this.ruleApplicationList);

		RuleApplicationList ruleApplications = getActiveQueryComponent().getRuleApplications();
		ruleApplicationList.setModel(ruleApplications.getFilteredModel());
	}

	private void hideRuleApplications() {

		this.jLabelRuleDescription.setText("");
		ruleApplicationList.setModel(new DefaultListModel<RuleApplication>());
		this.jScrollPane8.setViewportView(this.noRulesJLabel);
	}

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton applyChangesButton;
	private javax.swing.JButton checkWDButton;
	private javax.swing.JTextArea codeArea;
	private javax.swing.JFileChooser fileChooser;
	private javax.swing.JButton openStaticAnalysisButton;
	private javax.swing.JButton jButtonApplyRule;
	private javax.swing.JButton jButtonCreateRandomDataset;
	private javax.swing.JButton jButtonLoadDataset;
	private javax.swing.JButton jButtonLoadQuery;
	private javax.swing.JButton jButtonNewQuery;
	private javax.swing.JButton jButtonPreviewRule;
	private javax.swing.JButton runQueryButton;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel6;
	private javax.swing.JLabel jLabelRuleDescription;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JScrollPane jScrollPane3;
	private javax.swing.JScrollPane jScrollPane4;
	private javax.swing.JScrollPane jScrollPane5;
	private javax.swing.JScrollPane jScrollPane8;
	private javax.swing.JSplitPane jSplitPane1;
	private javax.swing.JList<DatasetComponent> listDatabases;
	private javax.swing.JList<QueryComponent> listQueries;
	private javax.swing.JTabbedPane lowerTabbedPane;
	private javax.swing.JTable resultsTable;
	private javax.swing.JList<RuleApplication> ruleApplicationList;
	private javax.swing.JTextArea statisticsArea;
	private javax.swing.JTabbedPane upperTabbedPane;
	private javax.swing.JList<WdConflictComponent> wdViolationsJList;
	private javax.swing.JScrollPane wdViolationsJScrollPane;
	// End of variables declaration//GEN-END:variables
	private javax.swing.JLabel wdViolationsJLabel;
	private javax.swing.JLabel noRulesJLabel;
}
