/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.gui.graph;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import spam.model.QueryComponent;
import spam.model.QueryComponent.TreeViewer;
import spam.model.SubtreeMatch;
import spam.model.TreePattern;
import spam.model.TreePattern.WdConflict;
import spam.model.TreePatternNode;
import spam.transformationRules.RuleApplication;

import com.hp.hpl.jena.shared.PrefixMapping;

/**
 *
 * @author Andres Letelier
 */
public class GraphicalTree extends JPanel {

	private static final long serialVersionUID = 1L;

	private int dimx, dimy;

	/*
	 * @Override public Dimension getSize() { return new Dimension(dimx, dimy); }
	 */

	GraphicalNode root;
	private final QueryComponent queryComponent;
	private NodeJPopupMenu popup;

	public GraphicalTree(QueryComponent queryComponent) {

		setBackground(Color.white);

		this.queryComponent = queryComponent;
		this.root = null;
		// register the mouse handler
		MouseHandler handler = new MouseHandler(this);
		this.addMouseListener(handler);
		this.addMouseMotionListener(handler);
	}

	public void setRoot(GraphicalNode root) {
		this.root = root;
		this.queryComponent.getTreePattern().setRoot(root.getNode());
	}

	public GraphicalNode getRoot() {
		return this.root;
	}

	private JPopupMenu getPopup() {

		if (this.popup == null) {
			this.popup = new NodeJPopupMenu("Alter Treenode");
		}
		return this.popup;
	}

	public void showPopup(GraphicalNode target, int x, int y) {
		this.getPopup();
		this.popup.setNode(this, target);
		this.popup.show(this, x, y);
	}

	@Override
	public void paintComponent(Graphics g) {

		super.paintComponent(g);

		if (this.root == null) {
			return;
		}
		if (!(g instanceof Graphics2D)) {
			return;
		}

		this.root.setPrefixes(this.queryComponent.getQuery().getPrefixMapping());

		this.root.recomputeOuterWidth(g);
		dimx = this.root.getOuterWidth() + 10;
		dimy = this.getImageHeight(g) + 10;

		boolean forSubsumption = false;
		Container cont;
		if (this.getParent() instanceof TreeViewer) {

			// we paint for the subsumption GUI
			cont = this.getParent().getParent();
			forSubsumption = true;
		} else {

			// we paint on the main screen
			cont = this.getParent();
		}

		if (cont != null && cont.getWidth() > dimx) {
			dimx = cont.getWidth();
		}

		this.setPreferredSize(new Dimension(dimx, dimy));
		this.root.paintNode((Graphics2D) g, 5, 10, forSubsumption);
		this.revalidate();

	}

	public void paintWithCounterexamples(Graphics g) {
		if (this.root != null) {
			this.root.setViewportMode(true);
			this.paintComponent(g);
			this.root.setViewportMode(false);
		}

	}

	public void init(TreePattern tp) {

		if (tp == null || tp.getRoot() == null) {
			this.root = null;
			return;
		}

		this.root = new GraphicalNode(tp.getRoot());
		this.root.init();
	}

	public void setHighlightVar(WdConflict conflict) {
		if (this.root == null) {
			return;
		}

		this.root.setWdHighlight(conflict);

	}

	public void highlightCounterexample(TreePattern subtree) {

		SubtreeMatch match = this.root.isSubtree(subtree.getRoot());

		if (match != null) {
			// tell all the nodes to highlight the match
			match.setCounterexampleNodes();
		}
	}

	public void unsetCounterexample() {
		if (this.root != null) {
			this.root.unsetCounterexample();
		}
	}

	public void setHighlightRuleApplicationNode(RuleApplication application) {
		this.root.setHighlightRuleApplicationNode(application);
	}

	/**
	 * checks if the mouse event happened on some node of the tree, and if this is the case the corresponding node is returned.
	 *
	 * @param point
	 * @return
	 */
	GraphicalNode findTargetNode(Point point) {
		if (this.root != null) {
			return this.root.mouseInside(point);
		}
		return null;
	}

	PrefixMapping getPrefixMapping() {
		return this.queryComponent.getQuery().getPrefixMapping();
	}

	/**
	 * tells the query component to reload the tree from the TreePattern and to redraw the screen
	 */
	protected void updateTree() {
		this.queryComponent.syncFromGraph();
	}

	public int getImageHeight(Graphics g) {
		if (this.root == null) {
			return 0;
		}
		return this.root.getMaxHeight(g);
	}

	public void unsetStatistics() {
		this.root.unsetStatistics();
	}

	public class NodeJPopupMenu extends JPopupMenu implements ActionListener {

		private static final long serialVersionUID = 1L;
		private GraphicalNode node;
		private GraphicalTree tree;
		private final JMenuItem editItem, deleteItem, addChildItem;

		public NodeJPopupMenu(String name) {
			super(name);
			this.editItem = this.add("edit");
			this.deleteItem = this.add("delete");
			this.addChildItem = this.add("add child node");

			this.editItem.addActionListener(this);
			this.deleteItem.addActionListener(this);
			this.addChildItem.addActionListener(this);
		}

		public void setNode(GraphicalTree tree, GraphicalNode node) {
			this.node = node;
			this.tree = tree;
		}

		@Override
		public void actionPerformed(ActionEvent ae) {

			if (ae.getSource() == this.editItem) {
				if (this.node.editContent(this.tree.getPrefixMapping())) {
					this.tree.updateTree();
				}
				// otherwise there was no change but only error

			} else if (ae.getSource() == this.deleteItem) {

				// ask if the children should be deleted or attached to the
				// parent

				Object[] options = { "Delete Node Only", "Delete Subtree rooted at Node" };
				int ans = JOptionPane.showOptionDialog(null, "Do you want to delete this node only, or the complete subtree " + "rooted at that node. If only this node is deleted, all of "
						+ "its children are attached as new children to the parent of " + "this node", "Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
				this.node.delete(ans);

				this.tree.updateTree();

			} else if (ae.getSource() == this.addChildItem) {

				TreePatternNode newPatternNode = new TreePatternNode("?s ?p ?o");
				GraphicalNode newGraphicalNode = new GraphicalNode(newPatternNode);
				newGraphicalNode.setPrefixes(this.tree.queryComponent.getQuery().getPrefixMapping());
				if (!newGraphicalNode.editContent(tree.getPrefixMapping())) {
					// we do not enter anything because no valid bgp was
					// specified
					JOptionPane.showMessageDialog(null, "Invalid BGP specified." + " No new node added");
				} else {
					// yes, we enter the new node
					this.node.addChild(newGraphicalNode);
					this.node.getNode().addChild(newPatternNode);
					newPatternNode.setParent(this.node.getNode());

					this.tree.updateTree();
				}
			}
		}
	}
}
