/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.gui.graph;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JOptionPane;

/**
 *
 * @author sebi
 */
public class MouseHandler extends MouseAdapter {

	GraphicalTree tree;
	GraphicalNode dragged;

	private int lastMouseX = -1;
	private int lastMouseY = -1;

	public MouseHandler(GraphicalTree tree) {
		this.tree = tree;
		this.dragged = null;
	}

	@Override
	public void mouseMoved(MouseEvent e) {
	}

	@Override
	public void mouseClicked(MouseEvent e) {
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// check if it is a popup trigger
		if (e.isPopupTrigger()) {
			this.checkForPopup(e);
		} else {

			GraphicalNode target = this.tree.findTargetNode(e.getPoint());
			if (target != null) {
				this.startDrag(target, e);
			}
		}
	}

	private void startDrag(GraphicalNode target, MouseEvent e) {
		this.lastMouseX = e.getX();
		this.lastMouseY = e.getY();

		this.dragged = target;
		// Graphics2D g2d = (Graphics2D)this.tree.getGraphics();
		// this.dragged.paintDragImage(g2d, e.getX(), e.getY());
		// g2d.dispose();
	}

	private void updateDrag(MouseEvent e) {

		Graphics2D g2d = (Graphics2D) this.tree.getGraphics();
		// System.out.println(this.tree.getBackground());
		g2d.setXORMode(Color.white);

		// first delete the old dragged element
		this.dragged.paintDragImage(g2d, this.lastMouseX, this.lastMouseY);

		// then draw the new one
		this.dragged.paintDragImage(g2d, e.getX(), e.getY());

		// finally save the new mouse positions
		this.lastMouseX = e.getX();
		this.lastMouseY = e.getY();

		g2d.dispose();
	}

	private void stopDrag() {

		this.lastMouseX = -1;
		this.lastMouseY = -1;

		this.dragged = null;

		// repaint in order to get rid of artefacts
		this.tree.repaint();
	}

	private void showRootMovedError() {
		JOptionPane.showMessageDialog(this.tree, "The root cannot be moved. Aborting.", "Root cannot be moved.", JOptionPane.ERROR_MESSAGE);
	}

	@Override
	public void mouseReleased(MouseEvent e) {

		// check if it is a popup trigger
		if (e.isPopupTrigger()) {
			this.checkForPopup(e);
		}

		if (this.dragged != null) {
			GraphicalNode target = this.tree.findTargetNode(e.getPoint());

			if (target != null && target != this.dragged) {

				Object[] options = { "Insert as parent", "Insert as sibling", "Insert as child", "Cancel" };
				int ans = JOptionPane.showOptionDialog(null, "How shall the node be inserted?", "Move node", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[2]);

				if (ans == 0) {
					// as parent
					if (!this.dragged.getNode().moveNode(target.getNode().getParent())) {
						this.showRootMovedError();
					} else {

						if (this.dragged.getNode().isRoot()) {
							this.dragged.getNode().addChild(this.tree.getRoot().getNode());
							this.tree.setRoot(this.dragged);
						}
						this.tree.updateTree();
					}
				} else if (ans == 1) {
					// as sibling
					if (target.getNode().isRoot()) {
						JOptionPane.showMessageDialog(this.tree, "The root cannot have a sibling. Aborting.", "Invalid Target", JOptionPane.ERROR_MESSAGE);
					} else {
						if (!this.dragged.getNode().moveNode(target.getNode().getParent())) {
							this.showRootMovedError();
						} else {
							this.tree.updateTree();
						}
					}

				} else if (ans == 2) {
					// as child
					if (!this.dragged.getNode().moveNode(target.getNode())) {
						this.showRootMovedError();
					} else {
						this.tree.updateTree();
					}
				}
			}
			this.stopDrag();
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {

		if (this.dragged != null) {
			this.updateDrag(e);
		}

	}

	private void checkForPopup(MouseEvent e) {

		GraphicalNode target = this.tree.findTargetNode(e.getPoint());

		if (target != null) {
			this.showPopup(e, target);
		}
	}

	private void showPopup(MouseEvent e, GraphicalNode target) {

		if (!(e.getSource() instanceof GraphicalTree)) {
			return;
		}

		GraphicalTree graph = (GraphicalTree) e.getSource();
		graph.showPopup(target, e.getPoint().x, e.getPoint().y);

	}

}
