/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.transformationRules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import spam.model.QueryComponent;
import spam.model.TreePattern;
import spam.model.TreePatternNode;

import com.hp.hpl.jena.graph.Triple;

/**
 * Deletion of redundant triples
 *
 * @author Andres Letelier
 */
public class R1Application extends RuleApplication {

	@Override
	public String type() {
		return "R1";
	}

	@Override
	public void apply() {
		if (this.resultIsWellDesignedFlag) {
			node.getBgp().getPattern().getList().removeAll(toDelete);
			if (node.getBgp().getPattern().isEmpty()) {
				node.delete();
			}
		} else {
			for (Triple t : toDelete) {
				this.node.addRedundantTriple(t);
			}
		}
	}

	// triples to be deleted
	private final List<Triple> toDelete;

	public R1Application(TreePatternNode where, List<Triple> toDelete) {
		this.node = where;
		this.toDelete = toDelete;
	}

	/**
	 * Returns all rule 1 applications in tree
	 *
	 * @param tree
	 * @return
	 */
	public static List<RuleApplication> findAll(TreePattern tree) {
		List<RuleApplication> rules = new ArrayList<RuleApplication>();
		findAllWorker(tree.getRoot(), new HashSet<Triple>(), rules);

		return rules;
	}

	/**
	 * Adds every rule 1 application to rules
	 *
	 * @param node
	 * @param seen
	 * @param rules
	 */
	private static void findAllWorker(TreePatternNode node, Set<Triple> seen, List<RuleApplication> rules) {
		// we need to create a new set to add triples; otherwise the recursion
		// goes crazy in parallel branches since triples are never removed from the
		// set when the function returns
		if (node.getBgp() == null) {
			// Union Node
			for (TreePatternNode child : node.getChildren()) {
				findAllWorker(child, new HashSet<Triple>(seen), rules);
			}
			return;
		}

		Set<Triple> seen2 = new HashSet<Triple>(seen);
		seen2.addAll(node.getBgp().getPattern().getList());

		List<Triple> toDelete = new ArrayList<Triple>();

		for (Triple t : node.getBgp().getPattern().getList()) {
			// check if we've already seen this node
			if (seen.contains(t) && !node.isRedundantTriple(t)) {
				toDelete.add(t);
			}
			seen2.add(t);
		}

		// if we have something to delete, we create a new rule
		if (!toDelete.isEmpty()) {
			rules.add(new R1Application(node, toDelete));
		}

		// backtrack into children
		for (TreePatternNode child : node.getChildren()) {
			findAllWorker(child, seen2, rules);
		}
	}

	public List<Triple> getDeleteTriples() {
		return this.toDelete;
	}

	@Override
	public String getDescription() {
		if (this.resultIsWellDesignedFlag) {
			return "<html>Delete the <font color=\"#0000ff\">highlighted triple" + (toDelete.size() > 1 ? " " : "s " + "</font></html>");
		} else {
			return "<html>Cannot delete the <font color=\"#0000ff\">highlighted triple" + (toDelete.size() > 1 ? " " : "s ") + "</font><br/>because the result would not be well-designed."
					+ "<br/>They are marked.</html>";
		}
	}

	@Override
	public void highlightNode(QueryComponent queryComponent) {
		queryComponent.setRuleApplicationNode(this);
		queryComponent.repaint();
	}
}
