/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.transformationRules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import spam.model.QueryComponent;
import spam.model.TreePattern;
import spam.model.TreePatternNode;

import com.hp.hpl.jena.sparql.algebra.OpVars;
import com.hp.hpl.jena.sparql.core.Var;

/**
 * Deletion of unproductive nodes
 * 
 * @author Andres Letelier
 */
public class R2Application extends RuleApplication {

	@Override
	public String type() {
		return "R2";
	}

	@Override
	public void apply() {
		node.mergeIntoChildren();
	}

	/**
	 * Returns all rule 2 applications in tree
	 * 
	 * @param tree
	 * @return
	 */
	public static List<RuleApplication> findAll(TreePattern tree) {
		List<RuleApplication> rules = new ArrayList<RuleApplication>();
		findAllWorker(tree.getRoot(), new HashSet<Var>(), rules);
		return rules;
	}

	private static void findAllWorker(TreePatternNode node, Set<Var> seen, List<RuleApplication> rules) {
		if (node.getBgp() == null) {
			// Union Node
			for (TreePatternNode child : node.getChildren()) {
				findAllWorker(child, new HashSet<Var>(seen), rules);
			}
			return;
		}
		// check for new variables
		Set<Var> vars = new HashSet<Var>(OpVars.mentionedVars(node.getBgp()));
		if (seen.containsAll(vars) && !node.isRoot()) {
			rules.add(new R2Application(node));
		}
		// add new variables to the set
		Set<Var> seen2 = new HashSet<Var>(seen);
		seen2.addAll(vars);

		for (TreePatternNode child : node.getChildren()) {
			findAllWorker(child, seen2, rules);
		}
	}

	public R2Application(TreePatternNode node) {
		this.node = node;
	}

	@Override
	public String getDescription() {
		if (this.node.hasChildren()) {
			return "<html>Merge the <font color=\"#0000ff\">highlighted node</font><br/>" + "into all its children</html>";
		} else {
			return "<html>Delete the <font color=\"#0000ff\">highlighted node</font></html>";
		}
	}

	@Override
	public void highlightNode(QueryComponent queryComponent) {
		queryComponent.setRuleApplicationNode(this);
		queryComponent.repaint();
	}
}
