/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.transformationRules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import spam.model.QueryComponent;
import spam.model.TreePattern;
import spam.model.TreePatternNode;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.sparql.algebra.OpVars;
import com.hp.hpl.jena.sparql.core.Var;

/**
 * Homomorphism upwards
 *
 * @author Andres Letelier
 */
public class R3Application extends RuleApplication {

	@Override
	public String type() {
		return "R3";
	}

	@Override
	public void apply() {
		node.mergeIntoParent();
	}

	public R3Application(TreePatternNode node) {
		this.node = node;
	}

	/**
	 * Returns all rule 3 applications in tree
	 *
	 * @param tree
	 * @return
	 */
	public static List<RuleApplication> findAll(TreePattern tree) {
		List<RuleApplication> rules = new ArrayList<RuleApplication>();
		findAllWorker(tree.getRoot(), new HashSet<Triple>(), new HashSet<Var>(), rules);
		return rules;
	}

	private static void findAllWorker(TreePatternNode node, Set<Triple> branch, Set<Var> branchVars, List<RuleApplication> rules) {
		if (node.getBgp() == null) {
			// Union Node
			for (TreePatternNode child : node.getChildren()) {
				findAllWorker(child, new HashSet<Triple>(branch), new HashSet<Var>(branchVars), rules);
			}
			return;
		}
		Set<Var> intersection = new HashSet<Var>(OpVars.mentionedVars(node.getBgp()));
		intersection.retainAll(branchVars);

		if (TransformationUtilities.checkHomomorphism(node.getBgp(), branch, intersection)) {
			rules.add(new R3Application(node));
		}

		// recurse into children
		Set<Triple> branch2 = new HashSet<Triple>(branch);
		Set<Var> branchVars2 = new HashSet<Var>(branchVars);
		branch2.addAll(node.getBgp().getPattern().getList());
		branchVars2.addAll(OpVars.mentionedVars(node.getBgp()));

		for (TreePatternNode child : node.getChildren()) {
			findAllWorker(child, branch2, branchVars2, rules);
		}
	}

	@Override
	public String getDescription() {
		return "<html>Merge the <font color=\"#0000ff\">highlighted node</font><br/>into its parent</html>";
	}

	@Override
	public void highlightNode(QueryComponent queryComponent) {
		queryComponent.setRuleApplicationNode(this);
		queryComponent.repaint();
	}
}
