/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.transformationRules;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import spam.model.QueryComponent;
import spam.model.TreePattern;
import spam.model.TreePatternNode;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.sparql.algebra.OpVars;
import com.hp.hpl.jena.sparql.core.Var;

/**
 * Parallelization
 * 
 * @author Andres Letelier
 */
public class R4Application extends RuleApplication {

	// the child to be paralelized
	TreePatternNode child;

	public R4Application(TreePatternNode node, TreePatternNode child) {
		this.node = node;
		this.child = child;
	}

	@Override
	public String type() {
		return "R4";
	}

	@Override
	public void apply() {
		node.getChildren().remove(child);
		child.setParent(node.getParent());
		node.getParent().getChildren().add(child);

		// TODO: check for quasi well-designed and undo if the condition fails
	}

	/**
	 * Returns all rule 4 applications in tree
	 *
	 * @param tree
	 * @return
	 */
	public static List<RuleApplication> findAll(TreePattern tree) {
		List<RuleApplication> rules = new ArrayList<RuleApplication>();
		findAllWorker(tree.getRoot(), new HashSet<Triple>(), new HashSet<Var>(), rules);
		return rules;
	}

	private static void findAllWorker(TreePatternNode node, Set<Triple> branch, Set<Var> branchVars, List<RuleApplication> rules) {
		if (node.getBgp() == null) {
			// Union Node
			for (TreePatternNode child : node.getChildren()) {
				findAllWorker(child, new HashSet<Triple>(branch), new HashSet<Var>(branchVars), rules);
			}
			return;
		}

		Set<Var> intersection = new HashSet<Var>(OpVars.mentionedVars(node.getBgp()));
		intersection.retainAll(branchVars);

		// these are for when we backtrack into children
		Set<Triple> branch2 = new HashSet<Triple>(branch);
		Set<Var> branchVars2 = new HashSet<Var>(branchVars);
		branch2.addAll(node.getBgp().getPattern().getList());
		branchVars2.addAll(OpVars.mentionedVars(node.getBgp()));

		// now we check and backtrack
		for (TreePatternNode child : node.getChildren()) {
			Set<Triple> branchAndChild = new HashSet<Triple>(branch);
			branchAndChild.addAll(child.getBgp().getPattern().getList());
			if (!node.isRoot() && TransformationUtilities.checkHomomorphism(node.getBgp(), branchAndChild, intersection)) {
				rules.add(new R4Application(node, child));
			}
			findAllWorker(child, branch2, branchVars2, rules);
		}
	}

	@Override
	public String getDescription() {
		return "<html>Make the <font color=\"#0000ff\">highlighted node</font><br/>a child of its grandparent</html>";
	}

	@Override
	public void highlightNode(QueryComponent queryComponent) {
		queryComponent.setRuleApplicationNode(this);
		queryComponent.repaint();
	}

	@Override
	public TreePatternNode getHighlightNode() {
		return this.child;
	}
}
