package subPrograms;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;

import spam.exception.PatternTreeFactoryException;
import spam.exception.UnsupportedOperatorForPatternTreeException;
import spam.model.TreePattern;
import spam.model.TreePatternFactory;
import spam.service.WellDesignedService;
import spam.transformationRules.R1Application;
import spam.transformationRules.R2Application;
import spam.transformationRules.R3Application;
import spam.transformationRules.R4Application;
import spam.transformationRules.RuleApplication;
import spam.transformationRules.TransformationUtilities;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryParseException;
import com.hp.hpl.jena.sparql.algebra.Algebra;
import com.hp.hpl.jena.sparql.algebra.Op;
import com.hp.hpl.jena.sparql.algebra.OpAsQuery;
import com.hp.hpl.jena.sparql.expr.ExprException;

/**
 * Used to manipulate queries from dbpedia
 *
 * @author Andres Letelier
 */
public class QueryParser {

	public static void main(String[] args) throws IOException {

		boolean oldmethod = true;

		// System.setIn(new FileInputStream("/Users/yo/lgd.all.usables"));
		// System.setOut(new PrintStream("/Users/yo/lgd.out"));

		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		int malformed = 0, total = 0, welldesigned = 0, r1 = 0, r2 = 0, r3 = 0, r4 = 0, rf = 0;
		String linea;
		while ((linea = in.readLine()) != null) {
			total++;
			if (total % 1000 == 0) {
				System.err.println(total);
			}
			String query;
			Query q = null;
			try {
				query = URLDecoder.decode(linea, "UTF-8");
				q = QueryFactory.create(query);

				Op op = Algebra.compile(q);

				if (!WellDesignedService.isWellDesigned(op)) {
					continue;
				}
				welldesigned++;
				TreePattern tree = TreePatternFactory.create(op);

				boolean print = false;
				if (oldmethod) {
					if (TransformationUtilities.R1check(tree)) {
						System.out.print("R1");
						r1++;
						print = true;
					}
					if (TransformationUtilities.R2(tree)) {
						System.out.print("R2");
						r2++;
						print = true;

					}
					if (TransformationUtilities.R3(tree)) {
						System.out.print("R3");
						r3++;
						print = true;
					}
					if (TransformationUtilities.RFilter(tree)) {
						System.out.print("RF");
						rf++;
						print = true;
					}
				} else {

					List<RuleApplication> rules = R1Application.findAll(tree);
					if (!rules.isEmpty()) {
						r1++;
						System.out.print("R1");
						print = true;
					}
					rules = R2Application.findAll(tree);
					if (!rules.isEmpty()) {
						r2++;
						while (!rules.isEmpty()) {
							rules.get(0).apply();
							rules = R2Application.findAll(tree);
						}
						System.out.print("R2");
						print = true;
					}

					rules = R3Application.findAll(tree);
					if (!rules.isEmpty()) {
						r3++;
						while (!rules.isEmpty()) {
							rules.get(0).apply();
							rules = R3Application.findAll(tree);
						}
						System.out.print("R3");
						print = true;
					}
					rules = R4Application.findAll(tree);
					if (!rules.isEmpty()) {
						r4++;
						System.out.print("R4");
						print = true;
					}
				}
				if (TransformationUtilities.RFilter(tree)) {
					System.out.print("RF");
					rf++;
					print = true;
				}

				if (print) {
					System.out.print("\t" + URLEncoder.encode(q.toString(), "UTF-8") + "\t");
					System.out.println(URLEncoder.encode(OpAsQuery.asQuery(tree.toOP()).toString(), "UTF-8"));
				}

			} catch (ExprException e) {
				malformed++;
				continue;
			} catch (QueryParseException e) {
				malformed++;
				continue;
			} catch (UnsupportedOperatorForPatternTreeException | PatternTreeFactoryException e) {
				if (!e.getMessage().contains("table unit") && e.getMessage().contains("OpJoin")) {
					System.err.println(e.toString());
					System.err.println(q);
				}
			} catch (IllegalArgumentException e) {
				malformed++;
				continue;
				// System.err.println(e.toString());
				// e.printStackTrace(System.err);
				// System.err.println(linea);
				//
				// System.exit(1);
			} catch (NullPointerException e) {
				System.err.println(q);
				continue;
			}

		}

		System.err.println("Total " + total);
		System.err.println("Malformed " + malformed);
		System.err.println("well-designed " + welldesigned);
		System.err.println("r1 " + r1);
		System.err.println("r2 " + r2);
		System.err.println("r3 " + r3);
		System.err.println("r4 " + r4);
		System.err.println("rf " + rf);
	}

	public static void checkRules(String filename) throws FileNotFoundException, IOException {
		BufferedReader in = new BufferedReader(new FileReader(filename));
		String linea;
		int r1 = 0, r2 = 0, tot = 0;
		while ((linea = in.readLine()) != null) {
			String query = URLDecoder.decode(linea, "UTF-8");
			Query q = QueryFactory.create(query);
			Op op = Algebra.compile(q);

			try {
				TreePattern tree = TreePatternFactory.create(op);
				if (TransformationUtilities.R1check(tree)) {
					r1++;
					System.out.println("R1:");
					System.out.println(query);
				}
				if (TransformationUtilities.R2(tree)) {
					r2++;
					System.out.println("R2:");
					System.out.println(query);
				}

			} catch (Exception e) {
				System.err.println(e);
				System.err.println(op);
				continue;
			} finally {

				tot++;
			}
		}
		in.close();
		System.out.println(tot + " " + r1 + " " + r2);

	}

	/**
	 * Outputs only well-designed queries
	 *
	 * @param filename
	 */
	public static void checkWellDesigned(String filename) throws IOException {
		BufferedReader in = new BufferedReader(new FileReader(filename));
		String linea;
		int tot = 0;
		FileWriter out = new FileWriter("salida.txt");
		while ((linea = in.readLine()) != null) {
			String query = URLDecoder.decode(linea, "UTF-8");
			Query q = QueryFactory.create(query);
			if (WellDesignedService.isWellDesigned(q)) {
				out.write(linea + "\n");
				tot++;
			}

		}
		in.close();
		out.close();
		System.out.println(tot);
	}

	/**
	 * Parses a url with an encoded query and outputs prettified, url-encoded, parseable queries
	 *
	 * @param filename
	 * @throws IOException
	 */
	public static void parseQueries(String filename) throws IOException {
		BufferedReader in = new BufferedReader(new FileReader(filename));
		String linea;
		int tot = 0;
		FileWriter out = new FileWriter("salida.txt");

		while ((linea = in.readLine()) != null) {
			tot++;
			if (tot % 10000 == 0) {
				System.err.println(tot);
			}
			// for each line, we extract the query
			String[] parts = linea.split(" ");
			String url = parts[parts.length - 1];
			url = url.substring(1, url.length() - 1);
			parts = url.split("&");
			String query = "";
			for (String parte : parts) {
				if (parte.startsWith("query=")) {
					query = parte.substring("query=".length());
					break;
				}
			}

			try {
				query = URLDecoder.decode(query, "UTF-8");
			} catch (IllegalArgumentException ex) {
				// sometimes the url is incorrectly encoded and cannot be
				// decoded
				continue;
			}

			// we try to parse the query, since we only really care about
			// actual working queries
			query = query.replace("\n", " ");
			Query q;
			try {
				q = QueryFactory.create(query);
			} catch (Exception ex) {
				// could not parse query
				continue;
			}

			out.write(URLEncoder.encode(q.toString(), "UTF-8") + "\n");
			if (tot % 100000 == 0) {
				out.flush();
			}
		}
		in.close();
		out.close();
	}
}
