/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.model;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import spam.exception.PatternTreeFactoryException;
import spam.exception.UnsupportedOperatorForPatternTreeException;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.sparql.algebra.Op;
import com.hp.hpl.jena.sparql.algebra.OpAsQuery;

/**
 *
 * @author Andres Letelier
 */
public class TreePatternTest {

	public TreePatternTest() {
	}

	/**
	 * Test of equals method, of class TreePattern.
	 */
	@Test
	public void testEquals() {
		String s1 = "prefix a: <http://localhost/> select * where {{?A a:name ?N OPTIONAL {?A a:email ?E}} OPTIONAL {?A a:webpage ?W}}";
		String s2 = s1.replace("?A", "?A1");

		// q1==q2, q1=/=q3
		Query q1 = QueryFactory.create(s1), q2 = QueryFactory.create(s1), q3 = QueryFactory.create(s2);

		assertTrue(q1.equals(q2));
		assertFalse(q1.equals(q3));
		assertFalse(q2.equals(q3));
	}

	/**
	 * Test of toOp method, of class TreePattern.
	 * 
	 * @throws PatternTreeFactoryException
	 */
	@Test
	public void testToOp() throws UnsupportedOperatorForPatternTreeException, PatternTreeFactoryException {
		// @formatter:off
        String query = "SELECT DISTINCT  *\n"
                + "WHERE\n"
                + "  { ?M <http://localhost/example/poi> <http://localhost/example/art-museum> .\n"
                + "    ?M <http://localhost/example/rating> <http://localhost/example/top>\n"
                + "    OPTIONAL\n"
                + "      { { ?M <http://localhost/example/location> ?L .\n"
                + "          ?H <http://localhost/example/location> ?L .\n"
                + "          ?H <http://localhost/example/category> <http://localhost/example/h>\n"
                + "          OPTIONAL\n"
                + "            { ?A <http://localhost/example/location> ?L .\n"
                + "              ?A <http://localhost/example/poi> ?P\n"
                + "            }\n"
                + "        }\n"
                + "        OPTIONAL\n"
                + "          { { ?H <http://localhost/example/nearby> ?M\n"
                + "              OPTIONAL\n"
                + "                { ?H <http://localhost/example/price> ?C }\n"
                + "            }\n"
                + "            OPTIONAL\n"
                + "              { ?H <http://localhost/example/rating> ?R }\n"
                + "          }\n"
                + "      }\n"
                + "  }\n"
                + "LIMIT   100\n";
        // @formatter:on
		TreePattern instance = TreePatternFactory.create(query);
		Op op = instance.toOP();
		assertEquals(query, OpAsQuery.asQuery(op).toString());
	}

	@Test
	public void whenTreePatternIsWellDesigned_thenWDCheckIsPositive() throws PatternTreeFactoryException {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE{"
				+ "{?a ?b ?c}"
				+ "UNION"
				+ "{{?a ?b ?c} OPTIONAL {?d ?e ?f} OPTIONAL {?i ?g ?h}}"
				+ "}");
		//@formatter:on
		// when
		TreePattern tp = TreePatternFactory.create(q);
		// then
		assertTrue(tp.checkWellDesigned());
		assertTrue(tp.isWellDesigned());
	}

	@Test
	public void whenTreePatternIsWellDesigned_thenQWDCheckIsPositive() throws PatternTreeFactoryException {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE{"
				+ "{?a ?b ?c}"
				+ "UNION"
				+ "{{?a ?b ?c} OPTIONAL {?d ?e ?f} OPTIONAL {?i ?g ?h}}"
				+ "}");
		//@formatter:on
		// when
		TreePattern tp = TreePatternFactory.create(q);
		// then
		assertTrue(tp.checkQuasiWellDesigned());
		assertTrue(tp.isQuasiWellDesigned());
	}

	@Test
	public void whenTreePatternIsQuasiWellDesigned_thenWDCheckIsNegative() throws PatternTreeFactoryException {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE"
				+ "{"
				+ "{?a ?b ?c}"
				+ " UNION "
				+ "{{?a ?b ?c} OPTIONAL {{?d ?e ?f} OPTIONAL {?a ?g ?h}}}"
				+ "}");
		//@formatter:on
		// when
		TreePattern tp = TreePatternFactory.create(q);
		// then
		assertTrue(!tp.checkWellDesigned());
		assertTrue(!tp.isWellDesigned());
	}

	@Test
	public void whenTreePatternIsQuasiWellDesigned_thenQWDCheckIsPositive() throws PatternTreeFactoryException {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE"
				+ "{"
				+ "{?a ?b ?c}"
				+ " UNION "
				+ "{{?a ?b ?c} OPTIONAL {{?d ?e ?f} OPTIONAL {?a ?g ?h}}}"
				+ "}");
		//@formatter:on
		// when
		TreePattern tp = TreePatternFactory.create(q);
		// then
		assertTrue(tp.checkQuasiWellDesigned());
		assertTrue(tp.isQuasiWellDesigned());
	}

	@Test
	public void whenTreePatternIsNotWellDesigned_thenWDandQWDisNegative() throws PatternTreeFactoryException {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE{"
				+ "{?a ?b ?c}"
				+ "UNION"
				+ "{{?a ?b ?c} OPTIONAL {?d ?e ?f} OPTIONAL {?d ?g ?h}}"
				+ "}");
		//@formatter:on
		// when
		TreePattern tp = TreePatternFactory.create(q);
		// then
		assertTrue(!tp.checkWellDesigned());
		assertTrue(!tp.isWellDesigned());
		assertTrue(!tp.checkQuasiWellDesigned());
		assertTrue(!tp.isQuasiWellDesigned());
	}
}
