package spam.service;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import spam.exception.UntransformableOperatorException;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;

/**
 * @author Andreas Taranetz
 */
public class OptNormalFormServiceTest {

	/**
	 * @author Andreas Taranetz
	 * 
	 *         P1 . (P2 . P3)
	 */
	@Test
	public void whenConjunctiveQuery_thenThenOptNFisPositive() {
		// when
		Query query = QueryFactory
				.create("PREFIX  dc10: <http://purl.org/dc/elements/1.0/> PREFIX  dc11: <http://purl.org/dc/elements/1.1/> SELECT  ?x ?y WHERE {{ ?book dc10:creator ?c } . {{ ?book dc10:title ?x } . { ?book dc11:title ?y }}}");
		// given
		boolean isInOptNormalForm = OptNormalFormService.isOptNormalForm(query);
		// then
		assertTrue(isInOptNormalForm);
	}

	/**
	 * @author Andreas Taranetz
	 * 
	 *         P1 UNION (P2 . P3)
	 */
	@Test
	public void whenTopLevelUnion_thenThenOptNFisPositive() {
		// when
		Query query = QueryFactory
				.create("PREFIX  dc10: <http://purl.org/dc/elements/1.0/> PREFIX  dc11: <http://purl.org/dc/elements/1.1/> SELECT  ?x ?y WHERE {{ ?book dc10:creator ?c } UNION {{ ?book dc10:title ?x } . { ?book dc11:title ?y }}}");
		// given
		boolean isInOptNormalForm = OptNormalFormService.isOptNormalForm(query);
		// then
		assertTrue(isInOptNormalForm);
	}

	/**
	 * @author Andreas Taranetz
	 * 
	 *         P1 UNION (P2 UNION P3)
	 */
	@Test
	public void whenNestedTopLevleUnion_thenThenOptNFisPositive() {
		// when
		Query query = QueryFactory
				.create("PREFIX  dc10: <http://purl.org/dc/elements/1.0/> PREFIX  dc11: <http://purl.org/dc/elements/1.1/> SELECT  ?x ?y WHERE {{ ?book dc10:creator ?c } UNION {{ ?book dc10:title ?x } UNION { ?book dc11:title ?y }}}");
		// given
		boolean isInOptNormalForm = OptNormalFormService.isOptNormalForm(query);
		// then
		assertTrue(isInOptNormalForm);
	}

	/**
	 * @author Andreas Taranetz
	 * 
	 *         P1 . (P2 UNION P3)
	 */
	@Test
	public void whenNestedUnion_thenOptNFisNegative() {
		// when
		Query query = QueryFactory
				.create("PREFIX  dc10: <http://purl.org/dc/elements/1.0/> PREFIX  dc11: <http://purl.org/dc/elements/1.1/> SELECT  ?x ?y WHERE {{ ?book dc10:creator ?c } . {{ ?book dc10:title ?x } UNION { ?book dc11:title ?y }}}");
		// given
		boolean isInOptNormalForm = OptNormalFormService.isOptNormalForm(query);
		// then
		assertFalse(isInOptNormalForm);
	}

	@Test
	public void whenNestedUnion_thenTransformedQueryIsInOptNF() throws UntransformableOperatorException {
		// when
		Query query = QueryFactory
				.create("PREFIX  dc10: <http://purl.org/dc/elements/1.0/> PREFIX  dc11: <http://purl.org/dc/elements/1.1/> SELECT  ?x ?y WHERE {{ ?book dc10:creator ?c } . {{ ?book dc10:title ?x } UNION { ?book dc11:title ?y }}}");
		// given
		boolean isInOptNormalFormOriginal = OptNormalFormService.isOptNormalForm(query);
		query = OptNormalFormService.getOptNormalForm(query);
		boolean isInOptNormalFormTransformed = OptNormalFormService.isOptNormalForm(query);
		// then
		assertFalse(isInOptNormalFormOriginal);
		assertTrue(isInOptNormalFormTransformed);
	}
}
