/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package spam.service;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import spam.exception.UnsupportedOperatorForSubsumptionException;

import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.sparql.algebra.Algebra;
import com.hp.hpl.jena.sparql.algebra.Op;

public class SubsumptionServiceTest {

	public SubsumptionServiceTest() {
	}

	@Test
	public void testIsEquivalent() throws UnsupportedOperatorForSubsumptionException {
		// given
		String s1 = "PREFIX prop: <http://localhost/properties/> select * where {{?A prop:name ?N OPTIONAL { ?A prop:email ?E}  }}";
		String s2 = "PREFIX prop: <http://localhost/properties/> select * where {{?A prop:name ?N OPTIONAL {?A prop:email ?E}} OPTIONAL {?A prop:webPage ?W}}";
		Op op1 = Algebra.compile(QueryFactory.create(s1));
		Op op2 = Algebra.compile(QueryFactory.create(s2));
		// when
		boolean op1IsEquivalentop1 = SubsumptionService.isEquivalent(op1, op1) == 0;
		boolean op1IsEquivalentop2 = SubsumptionService.isEquivalent(op1, op2) == 0;
		// then
		assertTrue(op1IsEquivalentop1);
		assertFalse(op1IsEquivalentop2);
	}

	/**
	 * Test of isSubsumed method, of class SubsumptionService.
	 * 
	 * @throws UnsupportedOperatorForSubsumptionException
	 */
	@Test
	public void testIsSubsumed() throws UnsupportedOperatorForSubsumptionException {
		// given
		String s1 = "PREFIX prop: <http://localhost/properties/> select * where {{?A prop:name ?N OPTIONAL { ?A prop:email ?E}  }}";
		String s2 = "PREFIX prop: <http://localhost/properties/> select * where {{?A prop:name ?N OPTIONAL {?A prop:email ?E}} OPTIONAL {?A prop:webPage ?W}}";
		Op op1 = Algebra.compile(QueryFactory.create(s1));
		Op op2 = Algebra.compile(QueryFactory.create(s2));
		// when
		boolean op1IsSubsumedByop2 = SubsumptionService.isSubsumed(op1, op2);
		boolean op1IsSubsumedByop1 = SubsumptionService.isSubsumed(op1, op1);
		boolean op2IsSubsumedByop1 = SubsumptionService.isSubsumed(op2, op1);
		// then
		assertTrue(op1IsSubsumedByop2);
		assertTrue(op1IsSubsumedByop1);
		assertFalse(op2IsSubsumedByop1);
	}

	@Test
	public void testIsSubsumedNestedOptional() throws UnsupportedOperatorForSubsumptionException {
		String s1 = "PREFIX p: <http://localhost/properties/> SELECT * WHERE { ?x p:p1 ?y OPTIONAL { ?x p:p2 ?z OPTIONAL { ?x p:p3 ?a }}}";
		String s2 = "PREFIX p: <http://localhost/properties/> SELECT * WHERE { { ?x p:p1 ?y OPTIONAL { ?x p:p2 ?z }} OPTIONAL { ?x p:p3 ?a }}";
		Op op1 = Algebra.compile(QueryFactory.create(s1));
		Op op2 = Algebra.compile(QueryFactory.create(s2));
		// when
		boolean op1IsSubsumedByop2 = SubsumptionService.isSubsumed(op1, op2);
		boolean op2IsSubsumedByop1 = SubsumptionService.isSubsumed(op2, op1);
		// then
		assertTrue(op1IsSubsumedByop2);
		assertFalse(op2IsSubsumedByop1);
	}

	@Test
	public void testIsSubsumedNestedOptionalAndUnion() throws UnsupportedOperatorForSubsumptionException {
		String s1 = "PREFIX p: <http://localhost/properties/> SELECT * WHERE {{?x p:p5 ?y} UNION { ?x p:p1 ?y OPTIONAL { ?x p:p2 ?z OPTIONAL { ?x p:p3 ?a }}}}";
		String s2 = "PREFIX p: <http://localhost/properties/> SELECT * WHERE {{?x p:p5 ?y} UNION {{?x p:p1 ?y OPTIONAL { ?x p:p2 ?z }} OPTIONAL { ?x p:p3 ?a }}}";
		Op op1 = Algebra.compile(QueryFactory.create(s1));
		Op op2 = Algebra.compile(QueryFactory.create(s2));
		// when
		boolean op1IsSubsumedByop2 = SubsumptionService.isSubsumed(op1, op2);
		boolean op2IsSubsumedByop1 = SubsumptionService.isSubsumed(op2, op1);
		// then
		assertTrue(op1IsSubsumedByop2);
		assertFalse(op2IsSubsumedByop1);
	}

	@Test
	public void testSubsumptionWithTopLevelUnion() throws UnsupportedOperatorForSubsumptionException {
		// given
		String s1 = "SELECT * WHERE { ?x ?y ?z . ?s ?p ?o }";
		String s2 = "SELECT * WHERE {{ ?a ?b ?c } UNION { ?x ?y ?z { ?s ?p ?o }}}";
		Op op1 = Algebra.compile(QueryFactory.create(s1));
		Op op2 = Algebra.compile(QueryFactory.create(s2));
		// when
		boolean op1IsSubsumedByop2 = SubsumptionService.isSubsumed(op1, op2);
		boolean op2IsSubsumedByop2 = SubsumptionService.isSubsumed(op1, op1);
		boolean op2IsSubsumedByop1 = SubsumptionService.isSubsumed(op2, op1);
		// then
		assertTrue(op1IsSubsumedByop2);
		assertTrue(op2IsSubsumedByop2);
		assertFalse(op2IsSubsumedByop1);
	}
}
