package spam.service;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import spam.exception.UntransformableOperatorException;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;

public class WellDesingedServiceTest {
	@Test
	public void whenQueryIsWellDesigned_thenWDCheckIsPositive() {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE{"
				+ "{?a ?b ?c}"
				+ "UNION"
				+ "{{?a ?b ?c} OPTIONAL {?d ?e ?f} OPTIONAL {?i ?g ?h}}"
				+ "}");
		//@formatter:on
		// when
		boolean isWD = WellDesignedService.isWellDesigned(q);
		// then
		assertTrue(isWD);
	}

	@Test
	public void whenQueryIsOnlyQuasiWellDesigned_thenWDCheckIsNegative() {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE"
				+ "{"
				+ "{{?a ?b ?c} OPTIONAL {{?d ?e ?f} OPTIONAL {?a ?g ?h}}}"
				+ " UNION "
				+ "{?a ?b ?c}"
				+ "}");
		//@formatter:on
		// when
		boolean isWD = WellDesignedService.isWellDesigned(q);
		// then
		assertFalse(isWD);
	}

	@Test
	public void whenQueryIsNotWellDesigned_thenWDCheckIsNegative() {
		// given
		//@formatter:off
		Query q = QueryFactory.create("SELECT * WHERE{"
				+ "{?a ?b ?c}"
				+ "UNION"
				+ "{{?a ?b ?c} OPTIONAL {?d ?e ?f} OPTIONAL {?d ?g ?h}}"
				+ "}");
		//@formatter:on
		// when
		boolean isWD = WellDesignedService.isWellDesigned(q);
		// then
		assertFalse(isWD);
	}

	@Test
	public void whenFilerQueryIsNotWellDesigned_thenWDCheckIsNegative() throws UntransformableOperatorException {
		// when
		//@formatter:off
		Query query = QueryFactory.create("PREFIX e: <http://example.com/> "
				+ "SELECT ?resource "
				+ "WHERE {"
				+ "{?resource e:FN ?name} "
				+ "OPTIONAL"
				+ "{?resource e:age ?age} "
				+ "FILTER (?age >= 22)"
				+ "}");
		//@formatter:on
		// given
		boolean isWellDesigned = WellDesignedService.isWellDesigned(query);
		assertFalse(isWellDesigned);
	}

	@Test
	public void whenNotWellFormedQueryIsWellDesigned_thenTransformedQueryIsWellDesigned() throws UntransformableOperatorException {
		// when
		//@formatter:off
		Query query = QueryFactory.create("PREFIX e: <http://example.com/> "
				+ "SELECT ?resource "
				+ "WHERE {"
				+ "{?resource e:age ?age} " // here is the difference
				+ "OPTIONAL"				// \/ /\ these are switched
				+ "{?resource e:FN ?name} " // here is the difference
				+ "FILTER (?age >= 22)"
				+ "}");
		//@formatter:on
		// given
		boolean isOriginalWellFormed = WellDesignedService.isWellDesigned(query);
		query = OptNormalFormService.getTreePatternConvertableForm(query);
		boolean isTransformedWellFormed = WellDesignedService.isWellDesigned(query);
		// then
		assertTrue(isOriginalWellFormed);
		assertTrue(isTransformedWellFormed);
	}

	@Test
	public void whenNotWellFormedUNIONQueryIsWellDesigned_thenTransformedQueryIsWellDesigned() throws UntransformableOperatorException {
		// when
		//@formatter:off
		Query query = QueryFactory.create("PREFIX e: <http://example.com/>"
				+ "SELECT ?resource WHERE "
				+ "{"
				+ "	{"
				+ "  {?name e:age ?age}"
				+ "	}"
				+ "	UNION"
				+ " {"
				+ "  {?resource e:age ?age}"
				+ "  OPTIONAL"
				+ "  {?resource e:FN ?name}"
				+ "  FILTER (?age >= \"22\")"
				+ "	}"
				+ "	FILTER (?age <= \"30\")"
				+ "}");
		//@formatter:on
		// given
		boolean isOriginalWellFormed = WellDesignedService.isWellDesigned(query);
		query = OptNormalFormService.getOptNormalForm(query);
		query = OptNormalFormService.getTreePatternConvertableForm(query);
		boolean isTransformedWellFormed = WellDesignedService.isWellDesigned(query);
		// then
		assertTrue(isOriginalWellFormed);
		assertTrue(isTransformedWellFormed);
	}
}
